/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.mmcomp.adom;

/*
 * ADOMQueryProcess.java
 *
 * Created on February 10, 2003, 1:04 PM
 */


import java.util.Iterator;
import java.util.ArrayList;
import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMNode;
import org.moremotion.adom.ADOMUtils;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.evaluator.MMSymbolResolver;
import org.moremotion.process.MemoryProcessRecord;
import org.moremotion.process.ProcessException;
import org.moremotion.process.ProcessRecord;
import org.moremotion.process.QueryProcess;
import org.moremotion.util.StringParser;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ADOMQueryProcess.java 51 2008-03-10 11:46:54Z erkan $
 */
public class ADOMQueryProcess extends QueryProcess {

  private ConfigParameter cpAdomName;
  private ConfigParameter cpScope;
  private ConfigParameter cpQuery;
  private ConfigParameter cpFieldsToInclude;

  private ArrayList colNames;
  private ArrayList colNamesAs;

  private MMSymbolResolver mmsr;
  private ADOM adom;
  private Iterator itr;

  public void prepare() throws ProcessException {

    if (logger.LEVEL1) {
      logger.print("Process started. " + callDef.getStepCaption());
    }

    try {
      cpAdomName = processConfig.getParameter("adomName");
      cpScope = processConfig.getParameter("adomScope");
      cpQuery = processConfig.getParameter("query");
      cpFieldsToInclude = processConfig.getParameter("fieldsToInclude");

    } catch (ConfigParameterException ex) {
      throw new ProcessException(ex);
    }

    if (logger.LEVEL2) {
      logger.print("PROCESS PARAMETERS " +
          "\n ADOM Name          : " + cpAdomName +
          "\n ADOM Scope         : " + cpScope +
          "\n ADOM Query Expr.   : " + cpQuery +
          "\n Fields to Include  : " + cpFieldsToInclude
          );
    }

    mmsr = request.getSymbolResolver();
  }

  private char getScope() throws ConfigParameterException{
    char scope = cpScope.resolve(request).charValue('2');
    switch (scope) {
      case '1': return 'r';
      case '2': return 's';
      case '3': return 'a';
    }
    return 's';
  }

  public void processRecord(ProcessRecord prec) throws ProcessException {

    try {

      if (logger.LEVEL2) {
        logger.print("Record Index : " + prec.getIndex() );
      }

      String adomName = cpAdomName.resolve(request).stringValue();

      if (logger.LEVEL2 && cpAdomName.hasMScript())  {
        logger.print("ADOM Name after resolving : " + adomName);
      }

      adom = ADOMUtils.findADOM(request,getScope() + ":" + adomName);

      if (adom == null) return;

      itr = adom.getRootNode().getNodes();

    } catch (Exception ex) {

      if (logger.LEVEL1) {
        logger.print("Exception occured. Message: " + ex.getMessage() );
      }
      throw new ProcessException(ex);
    }

  }


  public String[] getFieldNames() throws ProcessException {

    prepareFieldNames();
    String[] result = new String[colNamesAs.size()];
    colNamesAs.toArray(result);

    return result;
  }

  private void prepareFieldNames() throws ProcessException {

    String fieldList;
    try {
      fieldList = cpFieldsToInclude.resolve(request).stringValue("");
    } catch (ConfigParameterException ex) {
      throw new ProcessException(ex);
    }

    if (logger.LEVEL2 && cpFieldsToInclude.hasMScript()) {
      logger.print("Fields after resolving : " + fieldList);
    }

    colNames = new ArrayList();
    colNamesAs = new ArrayList();

    if (fieldList.equals("ALL_ADOM_FIELDS")) {

      if (logger.LEVEL2) {
        logger.print("ALL_ADOM_FIELDS requested");
      }
      ADOMNode item = adom.findNode("item");
      if (item != null) {
        Iterator it = item.getNodes();
        while (it.hasNext()) {
          ADOMNode node = (ADOMNode)it.next();
          String name = node.getName();
          colNames.add(name);
          colNamesAs.add(name);
        }
      }

    } else {

      StringParser sp = new StringParser(fieldList,',');

      while (sp.hasMore()) {
        String fieldName = sp.nextItem().trim();
        if (logger.LEVEL3) logger.print("  Field Name : " + fieldName);
        int colpos = fieldName.indexOf(':');
        if (colpos > -1) {
          colNames.add( fieldName.substring(colpos+1).trim() );
          colNamesAs.add( fieldName.substring(0,colpos).trim() );
        } else {
          colNames.add(fieldName);
          colNamesAs.add(fieldName);
        }
      }
    }

  }


  public MemoryProcessRecord getNextResultRecord() throws ProcessException {

    if (itr == null) return null;

    if (colNames == null) prepareFieldNames();

    if (itr.hasNext()) {
      ADOMNode item = (ADOMNode)itr.next();

      // Skip non "item" nodes
      while (itr.hasNext() && !item.getName().equals("item")) { 
        item = (ADOMNode)itr.next();
      }
    
      if (item.getName().equals("item")) { 

        mmsr.setADOMNode(item);

        boolean doit;
        try {
          doit = cpQuery.isEmpty() || cpQuery.resolve(request).expressionValue().evalBool(mmsr);
        } catch (Exception ex) {
          throw new ProcessException(ex);
        }

        if (doit) {
          MemoryProcessRecord result = new MemoryProcessRecord();
          result.fields = new String[colNames.size()];

          for (int ci = 0; ci < colNames.size(); ci++) {

            String colName = (String)colNames.get(ci);
            String colNameAs = (String)colNamesAs.get(ci);
            String colValue = null;
            if (colName.equals("id"))  {
              colValue = item.getAttribute("id");
            } else {
              colValue = item.getNodeValue(colName);
            }

            if (logger.LEVEL3) logger.print("  " + fixedWidth(colNameAs) + " : " + colValue);

            result.fields[ci] = colValue;
          }

          return result;

        } else {
          return getNextResultRecord();
        }
      }

    }

    return null;
  }

  public void finalizeProcess() throws ProcessException {
  }


  private String fixedWidth(String s) {
    return (s + "                   ").substring(0,20);
  }

}
