/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * AML.java
 *
 * Created on 06 August 2004 Friday, 17:31
 */

package org.moremotion.mmcomp.adom;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMNode;
import org.moremotion.clang.CLException;
import org.moremotion.clang.CustomLanguage;
import org.moremotion.clang.FunctionSyntax;
import org.moremotion.clang.LanguageSyntax;
import org.moremotion.clang.ParameterSyntax;
import org.moremotion.evaluator.ADOMName;
import org.moremotion.evaluator.Evaluable;
import org.moremotion.evaluator.Expression;
import org.moremotion.evaluator.MMSymbolResolver;
import org.moremotion.evaluator.Symbol;
import org.moremotion.evaluator.EvalUtils;
import org.moremotion.process.ProcessRecord;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.adom.ADOMUtils;
import org.moremotion.util.Logger;


/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: AML.java 51 2008-03-10 11:46:54Z erkan $
 */
public class AML extends CustomLanguage {

  private final static int __Get = 1;
  private final static int __Delete = 3;
  private final static int __Clear = 4;
  private final static int __ToXML = 5;
  private final static int __Save = 6;
  private final static int __Load = 7;
  private final static int __Insert = 11;
  private final static int __Update = 12;
  private final static int __Sort = 13;
  private final static int __Copy = 14;
  private final static int __ParseXML = 15;


  private static LanguageSyntax lang;
  private MMSymbolResolver sr;
  private MoreMotionRequest request;
  private ProcessRecord prec = null;
//  private FormProcessBlock pb = null;
  private int impliedSymbolCount = 0;
  private int symbolCount = 0;
  private int asterixIndex = -1;
  private boolean allRequested = false;
  private boolean compatible = false;

  static {

    lang = new LanguageSyntax("AML", new FunctionSyntax[] {

      new FunctionSyntax(__Get, "Get", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_ADOM, "ADOM Name", true),
        new ParameterSyntax( ParameterSyntax.PT_LIST, "Value List",  true),
        new ParameterSyntax( ParameterSyntax.PT_LIST, "Target Symbol List",  true),
        new ParameterSyntax( ParameterSyntax.PT_TEST_EXPRESSION, "Match Expression",  true)
      }
      ),
      new FunctionSyntax(__Insert, "Insert", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_ADOM, "ADOM Name", true),
        new ParameterSyntax( ParameterSyntax.PT_LIST, "Node List",  true),
        new ParameterSyntax( ParameterSyntax.PT_LIST, "Value List",  true),
        new ParameterSyntax( ParameterSyntax.PT_TEST_EXPRESSION, "Match Expression",  false),
        new ParameterSyntax( ParameterSyntax.PT_BOOLEAN, "Replace",  false)
      }
      ),
      new FunctionSyntax(__Update, "Update", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_ADOM, "ADOM Name", true),
        new ParameterSyntax( ParameterSyntax.PT_LIST, "Node List",  true),
        new ParameterSyntax( ParameterSyntax.PT_LIST, "Value List",  true),
        new ParameterSyntax( ParameterSyntax.PT_TEST_EXPRESSION, "Match Expression",  false)
      }
      ),
      new FunctionSyntax(__Delete, "Delete", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_ADOM, "ADOM Name", true),
        new ParameterSyntax( ParameterSyntax.PT_TEST_EXPRESSION, "Match Expression",  false)
      }
      ),
      new FunctionSyntax(__Clear, "Clear", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_ADOM, "ADOM Name", true)
      }
      ),
      new FunctionSyntax(__Load, "Load", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_ADOM, "ADOM Name", true),
        new ParameterSyntax( ParameterSyntax.PT_STRING, "File Name", true)
      }
      ),
      new FunctionSyntax(__Save, "Save", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_ADOM, "ADOM Name", true),
        new ParameterSyntax( ParameterSyntax.PT_STRING, "File Name", true)
      }
      ),
      new FunctionSyntax(__ToXML, "ToXML", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_ADOM, "ADOM Name", true),
        new ParameterSyntax( ParameterSyntax.PT_SYMBOL, "Symbol Name", true)
      }
      ),
      new FunctionSyntax(__ParseXML, "ParseXML", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_ADOM, "ADOM Name", true),
        new ParameterSyntax( ParameterSyntax.PT_STRING, "XML String", true)
      }
      ),
      new FunctionSyntax(__Sort, "Sort", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_ADOM, "ADOM Name", true),
        new ParameterSyntax( ParameterSyntax.PT_STRING, "Sort Definition", true)
      }
      ),
      new FunctionSyntax(__Copy, "Copy", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_ADOM, "ADOM Name", true),
        new ParameterSyntax( ParameterSyntax.PT_ADOM, "New ADOM Name", true)
      }
      )
    }
    );

  }

  public AML(MMSymbolResolver sr, Logger parent) {
    super(sr.getRequest(), parent);
    this.sr = sr;
    this.request = sr.getRequest();
  }

//  public void setProcessBlock(FormProcessBlock pb) {
//    this.pb = pb;
//  }

  public void setProcessRecord(ProcessRecord prec) {
    this.prec = prec;
  }


  public int execute(String str) throws CLException  {
    return super.execute(str);
  }

  public int executeFunction(int funcid, Object[] parms) throws CLException {

    int result = 0;

    try {
      switch (funcid) {
        case __Insert : result = Insert( (ADOMName)parms[0], (Object[])parms[1], (Object[])parms[2], (Expression)parms[3], (Boolean)parms[4]); break;
        case __Update : result = Update( (ADOMName)parms[0], (Object[])parms[1], (Object[])parms[2], (Expression)parms[3]) ; break;
        case __Delete : result = Delete( (ADOMName)parms[0], (Expression)parms[1] ); break;
        case __Get    : result = Get(    (ADOMName)parms[0], (Object[])parms[1], (Object[])parms[2], (Expression)parms[3]) ; break;
        case __Clear  : result = Clear(  (ADOMName)parms[0]);  break;
        case __Load   : result = Load(   (ADOMName)parms[0], (String)parms[1] ); break;
        case __Save   : result = Save(   (ADOMName)parms[0], (String)parms[1] ); break;
        case __ToXML  : result = ToXML(  (ADOMName)parms[0], (Symbol)parms[1] ); break;
        case __ParseXML : result = ParseXML(  (ADOMName)parms[0], (String)parms[1] ); break;
        case __Sort   : result = Sort(  (ADOMName)parms[0], (String)parms[1] ); break;
        case __Copy   : result = Copy(  (ADOMName)parms[0], (ADOMName)parms[1] ); break;
      }
    } catch (Exception e) {
      throw new CLException(e);
    }

    return result;
  }

  public LanguageSyntax getSyntax() {
    return lang;
  }


  private int ToXML(ADOMName adomName, Symbol sym) throws AMLException {

    ADOM adom = ADOMUtils.findADOM(request, adomName.toString());

    if (adom != null) {
      try {
        sr.setValue(sym.toString(), adom.toXml(null));
      } catch (Exception e) {
        throw new AMLException(e);
      }
    }

    return 0;

  }

  private int ParseXML(ADOMName adomName, String xml) throws AMLException {

    ADOM adom = ADOMUtils.findADOM(request, adomName.toString());

    if (adom == null) {
      switch (adomName.getScope()) {
        case 'r': adom = request.newRequestADOM(adomName.getName()); break;
        case 's': adom = request.newSessionADOM(adomName.getName()); break;
        case 'a': adom = request.newApplicationADOM(adomName.getName()); break;
      }
    } else {
      adom.clear();
    }

    if (xml != null && xml.length() > 0) {
      try {
        adom.parseXML(xml);
      } catch (Exception e) {
        throw new AMLException(e);
      }
    }

    return 0;


  }


  private int Sort(ADOMName adomName, String sortDefinition) throws AMLException {

    ADOM adom = ADOMUtils.findADOM(request, adomName.toString());

    if (adom != null) {
      try {
        adom.sort(sortDefinition);
      } catch (Exception e) {
        throw new AMLException(e);
      }
    }

    return 0;

  }


  private int Load(ADOMName adomName, String fileName) throws AMLException {

    ADOM adom = ADOMUtils.findADOM(request, adomName.toString());

    if (adom == null) {
      switch (adomName.getScope()) {
        case 'r': adom = request.newRequestADOM(adomName.getName()); break;
        case 's': adom = request.newSessionADOM(adomName.getName()); break;
        case 'a': adom = request.newApplicationADOM(adomName.getName()); break;
      }
    } else {
      adom.clear();
    }

    File f = request.getRealFile(fileName);
    if (f.exists()) {
      try {
        adom.loadFromFile(f);
      } catch (Exception e) {
        throw new AMLException(e);
      }
    }

    return 0;

  }


  private int Save(ADOMName adomName, String fileName) throws AMLException {

    ADOM adom = ADOMUtils.findADOM(request, adomName.toString());
    File f = request.getRealFile(fileName);
    try {
      adom.saveToFile(f);
    } catch (Exception e) {
      throw new AMLException(e);
    }
    if (logger.LEVEL2) logger.print("XML is saved to file " + f.getAbsolutePath());

    return 0;

  }


  private int Get(ADOMName adomName, Object[] vlist, Object[] slist, Expression where) throws AMLException {

    ADOM adom = ADOMUtils.findADOM(request, adomName.toString());
    if (adom == null) {
      throw new AMLException("ADOM_MISSING",new String[]{adomName.getName()},null);
    }

    ADOMNode item = selectNode(adom, where, true);

    Evaluable[] values = getValues(vlist, item);
    Symbol[] symbols = getSymbols(slist);

    if (symbols.length != values.length) {
      throw new AMLException("AML_EXCEPTION:SYMBOL_AND_VALUE_ARRAYS_INCONSISTENT");
    }

    try {

      sr.setADOMNode(item);
      for (int j = 0; j < symbols.length; j++) {
        Symbol s = symbols[j];
        Evaluable e = values[j];
        if (s.getName().equals("?")) {
          sr.setValue(s.getSTS() + ":" + ((Symbol)e).getName(), EvalUtils.toStr(e.evaluate(sr)));
        } else {
          String val = EvalUtils.toStr(e.evaluate(sr));
          sr.setAdomNodeProcessingMode(MMSymbolResolver.ADOM_NODE_PROCESSING_MODE_IGNORE);
          sr.setValue(s.toString(), val);
          sr.resetAdomNodeProcessingMode();
        }

      }
      sr.setADOMNode(null);

    } catch (Exception e) {
      throw new AMLException(e);
    }

    return 0;

  }


  private int Delete(ADOMName adomName, Expression where) throws AMLException { 

    ADOM adom = ADOMUtils.findADOM(request, adomName.toString());
    if (adom == null) {
      throw new AMLException("ADOM_MISSING",new String[]{adomName.getName()},null);
    }

    ADOMNode[] items = findItems(adom, where);

    try {

      for (int i = 0; i < items.length; i++) {
        ADOMNode item = items[i];
        adom.removeNode(item);
      }
    } catch (Exception e) {
      throw new AMLException(e);
    }

    return 0;

  }

  private int Copy(ADOMName adomName, ADOMName newADOMName) throws AMLException { 

    ADOM adom = ADOMUtils.findADOM(request, adomName.toString());
    if (adom == null) {
      throw new AMLException("ADOM_MISSING",new String[]{adomName.getName()},null);
    }
    
    ADOM newADOM = ADOMUtils.findADOM(request, newADOMName.toString());
    if (newADOM != null) {
      Clear(newADOMName);
    }
    
    try {
      newADOM = (ADOM) adom.clone();
    } catch (CloneNotSupportedException ex) {
      throw new AMLException(ex);
    }
      switch (newADOMName.getScope()) {
        case 'r': request.setAttribute("ADOM_" + newADOMName.getName(),newADOM);
        case 's': request.getSession().setAttribute("ADOM_" + newADOMName.getName(),newADOM);
        case 'a': request.getContext().setAttribute("ADOM_" + newADOMName.getName(),newADOM);
      }
    
    return 0;

  }

  private int Insert(ADOMName adomName, Object[] slist, Object[] vlist, Expression match, Boolean replace) throws AMLException {

    ADOM adom = ADOMUtils.findADOM(request, adomName.toString());
    if (adom == null) {
      char scope = adomName.getScope();
      switch (adomName.getScope()) {
        case 'r': adom = request.newRequestADOM(adomName.getName()); break;
        case 's': adom = request.newSessionADOM(adomName.getName()); break;
        case 'a': adom = request.newApplicationADOM(adomName.getName()); break;
      }
    }

    Evaluable[] values = getValues(vlist, null);
    Symbol[] symbols = getSymbols(slist);

    if (symbols.length != values.length) {
      throw new AMLException("AML_EXCEPTION:SYMBOL_AND_VALUE_ARRAYS_INCONSISTENT");
    }

    try {

      ADOMNode item = null;

      if (match != null) {
        item = selectNode(adom,match,false);
        if (item != null) {
          if (replace != null && replace.booleanValue() == false) return 0;
        }
      }
      if (item == null) {
        item = new ADOMNode("item");
        adom.addNode(item);   
        
        int nodeid = 0;
        try {
          nodeid = Integer.parseInt(adom.getNodeValue("incr"));
          nodeid++;
        } catch (Exception e) {};
        adom.setNodeValue("incr","" + nodeid);
        item.addAttribute("id","" + nodeid);      
        
      }

      updateItem(item,symbols,values);

      sr.setADOMNode(null);

    } catch (Exception e) {
      throw new AMLException(e);
    }
    return 0;

  }


  private int Update(ADOMName adomName, Object[] slist, Object[] vlist, Expression where) throws AMLException {

    ADOM adom = ADOMUtils.findADOM(request, adomName.toString());
    if (adom == null) {
      throw new AMLException("ADOM_MISSING",new String[]{adomName.getName()},null);
    }

    ADOMNode[] items = findItems(adom, where);

    Evaluable[] values = getValues(vlist, null);
    Symbol[] symbols = getSymbols(slist);

    if (symbols.length != values.length) {
      throw new AMLException("AML_EXCEPTION:SYMBOL_AND_VALUE_ARRAYS_INCONSISTENT");
    }

    try {

      for (int i = 0; i < items.length; i++) {
        ADOMNode item = items[i];
        updateItem(item,symbols,values);
      }

    } catch (Exception e) {
      throw new AMLException(e);
    }

    return 0;
  }


  private ADOMNode cloneItem(ADOMNode item) {
    ADOMNode clone = new ADOMNode("item");
    Iterator itr = item.getNodes();
    while (itr.hasNext()) {
      ADOMNode snode = (ADOMNode)itr.next();
      ADOMNode tnode = new ADOMNode(snode.getName(),snode.getValue());
      clone.addNode(tnode);
    }
    return clone;
  }

  private void updateItem(ADOMNode item, Symbol[] symbols, Evaluable[] values) throws Exception {

    ADOMNode clone = cloneItem(item);
    sr.setADOMNode(clone);
    for (int j = 0; j < symbols.length; j++) {
      Symbol s = symbols[j];
      if (s.getName().equals("?")) {
        Symbol e = (Symbol)values[j];
        // if (!overriden(e.getName(),slist))
        {
          sr.setAdomNodeProcessingMode(MMSymbolResolver.ADOM_NODE_PROCESSING_MODE_LAST);
          String val = EvalUtils.toStr(e.evaluate(sr));
          sr.resetAdomNodeProcessingMode();
          // item.setNodeValue(e.getName(), val );
          setItemNodeValue(item, e.getName(), val );
        }
      } else {
        Evaluable e = values[j];
        sr.setAdomNodeProcessingMode(MMSymbolResolver.ADOM_NODE_PROCESSING_MODE_LAST);
        String val = EvalUtils.toStr(e.evaluate(sr));
        sr.resetAdomNodeProcessingMode();
        // item.setNodeValue(s.getName(), val );
        setItemNodeValue(item, s.getName(), val );
      }

    }
    sr.setADOMNode(null);

  }

  private void setItemNodeValue(ADOMNode item, String sym, String value) {
    if (sym.charAt(0) == '@') {
      item.addAttribute(sym.substring(1), value);
    } else {
      item.setNodeValue(sym, value);
    }
  }

  private boolean overriden(String name, Object[] slist) {
    for (int i = 0; i < slist.length; i++) {
      Symbol s = (Symbol)slist[i];
      if (s.getName().equals(name)) {
        return true;
      }
    }
    return false;
  }

  private int Clear(ADOMName adomName) throws AMLException {

    char scope = adomName.getScope();

    switch (adomName.getScope()) {
      case 'r': request.removeRequestADOM(adomName.getName()); break;
      case 's': request.removeSessionADOM(adomName.getName()); break;
      case 'a': request.removeApplicationADOM(adomName.getName()); break;
    }

    return 0;
  }

  private Evaluable[] getValues(Object[] vlist, ADOMNode item) throws AMLException {

    boolean already = false;
    ArrayList a = new ArrayList();
    asterixIndex = -1;
    symbolCount = 0;
    impliedSymbolCount = 0;

    for (int i = 0; i < vlist.length; i++) {

      Object o = vlist[i];
      boolean done = false;

      if (o instanceof Symbol && ((Symbol)o).getName().equals("*")) {
        allRequested = true;

        if (already) {
          throw new AMLException("AML_EXCEPTION:ASTERIX_USED_MORE_THAN_ONCE");

        } else {

          asterixIndex = i;
          already = true;

          if (item != null) {
            Iterator itr = item.getNodes();
            while (itr.hasNext()) {
              ADOMNode n = (ADOMNode)itr.next();
              impliedSymbolCount++;
              symbolCount++;
              a.add( new Symbol(n.getName()) );
            }

          } else {

            if (prec != null) {
              String[] fnames = prec.getBlock().getFieldNames();
              for (int x = 0; x < fnames.length; x++) {
                String sn = fnames[x];
                impliedSymbolCount++;
                symbolCount++;
                a.add( new Symbol(sn) );
              }
//              if (prec != null) {
//                e = prec.getCustomFieldNames();
//                if (e != null) {
//                  while (e.hasMoreElements()) {
//                    String sn = (String)e.nextElement();
//                    impliedSymbolCount++;
//                    symbolCount++;
//                    v.add( new Symbol(sn) );
//                  }
//                }
//              }
            }
          }
        }
      } else {
        symbolCount++;
        a.add((Evaluable)o);
      }

    }

    Evaluable[] result = new Evaluable[a.size()];
    a.toArray(result);
    return result;

  }

  private Symbol[] getSymbols(Object[] slist) throws AMLException {

    boolean already = false;
    ArrayList a = new ArrayList();

    for (int i = 0; i < slist.length; i++) {

      Object o = slist[i];
      if (!(o instanceof Symbol)) {
        throw new AMLException("AML_EXCEPTION:INVALID_SYMBOL",new String[]{o.toString()},null); 
      }

      Symbol sym = (Symbol)o;

      if (sym.getName().equals("*")) {
        if (already) {
          throw new AMLException("AML_EXCEPTION:ASTERIX_USED_MORE_THAN_ONCE");
        } else {
          already = true;
          if (impliedSymbolCount > 0 && i != asterixIndex) {
            throw new AMLException("AML_EXCEPTION:ASTERIX_POSITIONS_MISMATCH");
          } else {
            int cnt = impliedSymbolCount > 0 ? impliedSymbolCount : symbolCount;
            for (int j = 0; j < cnt; j++) {
              a.add( new Symbol("v:?")  );
            }
          }
        }
      } else {
        a.add( sym );
      }

    }

    Symbol[] result = new Symbol[a.size()];
    a.toArray(result);
    return result;
  }

  private ADOMNode selectNode(ADOM adom, Expression where, boolean create) throws AMLException {

    Iterator i = adom.getNodes();
    while (i.hasNext()) {
      ADOMNode item = (ADOMNode)i.next();
      if (item.getName().equals("item")) {
        sr.setADOMNode(item);
        try {
          if (where == null) {
            return item;
          } else {
            if (where.evalBool(sr)) {
              return item;
            }
          }
        } catch (Exception e) {
          throw new AMLException(e);
        }
      }
    }
    if (create) return new ADOMNode("item");
    return null;
  }

  private ADOMNode[] findItems(ADOM adom, Expression where) throws AMLException {

    ArrayList a = new ArrayList();
    Iterator i = adom.getNodes();
    while (i.hasNext()) {
      ADOMNode item = (ADOMNode)i.next();
      if (item.getName().equals("item")) {
        sr.setADOMNode(item);
        try {
          if (where == null) {
            a.add(item);
          } else {
            if (where.evalBool(sr)) {
              a.add(item);
            }
          }
        } catch (Exception e) {
          throw new AMLException(e);
        }
      }
    }

    sr.setADOMNode(null);

    ADOMNode[] result = new ADOMNode[a.size()];
    a.toArray(result);

    return result;
  }


}