/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * AjaxService.java
 *
 * Created on 06 December 2004 Monday, 17:50
 */

package org.moremotion.mmcomp.ajax;

import java.io.IOException;
import org.moremotion.action.ActionServiceContext;
import org.moremotion.action.ActionService;
import org.moremotion.page.CurrentPage;
import org.moremotion.process.ExecutionManager;
import org.moremotion.process.ProcessCommand;
import org.moremotion.process.ProcessCommandManager;
import org.moremotion.servlet.ServiceException;
import org.moremotion.servlet.MoreMotionRequest;


/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: AjaxProcessManager.java 172 2008-07-10 11:48:07Z erkan $
 */
public class AjaxProcessManager implements ActionService {
  
  public static void addResponse(MoreMotionRequest request, AjaxResponse response) {
    MultiPartAjaxResponse mpar = (MultiPartAjaxResponse)request.getAttribute("ajax.http.response");
    if (mpar != null) {
      mpar.addResponse(response);
    }
  }
  
  public void doService(ActionServiceContext asc) throws ServiceException, IOException {
    
    MoreMotionRequest request = asc.getRequest();
    MultiPartAjaxResponse mpar = new MultiPartAjaxResponse(request);
    
    request.setAttribute("ajax.http.response",mpar);
    
    try {
      
      String cmdName = request.getParameter("_cmd");
      String scope = request.getParameter("_scope");
      String originPage = request.getOriginPage();
      
      String fullCommandName = cmdName;
      if (scope != null && scope.trim().length() > 0) {
        fullCommandName = scope + "." + fullCommandName;
      }
      if (originPage != null && originPage.trim().length() > 0) {
        fullCommandName = originPage + "." + fullCommandName;
      }
      
      // Config config = ConfigManager.getConfig("ajaxProcessCommand",fullCommandName);
      ProcessCommand cmd = ProcessCommandManager.getAjaxProcessCommand(fullCommandName);
      if (cmd == null) {
        throw new AjaxException("AJAX_PROCESS_COMMAND_MISSING",new String[]{cmdName, scope, originPage},null);
      }
      
      ExecutionManager em = ExecutionManager.getExecutionManager(cmd.getTemplateName(request),request.getOriginPage(),request,cmd.getDebugLevel(request));
      int cc = em.execute();
      if (cc >= 12) {
        mpar.buildErrorFromMesageAdoms();
        return;
      }
      mpar.build();
      
    } catch (Exception ex) {
      mpar.buildError(ex);
      
    } finally {
      mpar.flush();
      
    }
    
  }
  
}
