/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * RefreshAreasProcess.java
 *
 * Created on 05 September 2007 Wednesday, 23:10
 *
 */

package org.moremotion.mmcomp.ajax;

import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.config.ConfigurationException;
import org.moremotion.process.Process;
import org.moremotion.process.ProcessException;
import org.moremotion.process.ProcessRecord;
import org.moremotion.util.XString;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: RefreshAreasProcess.java 51 2008-03-10 11:46:54Z erkan $
 */
public class RefreshAreasProcess extends Process {
  
  private ConfigParameter sourceAreas;
  private ConfigParameter targetAreas;
  private ConfigParameter responseHandler;
  
  public void prepare() throws ProcessException {
    
    if (logger.LEVEL1) {
      logger.printres("*PROCESS_STARTED", callDef.getStepName());
    }
    
    try {
      
      sourceAreas = processConfig.getParameter("sourceAreas");
      targetAreas = processConfig.getParameter("targetAreas");
      responseHandler = processConfig.getParameter("responseHandler");
      
      if (logger.LEVEL2) {
        logger.printres("*PROCESS_PARAMETERS_LINE");
        logger.printres("REFRESH_AREAS_PROCESS_PARAMETERS", sourceAreas.toString(), targetAreas.toString(), responseHandler.toString() );
      }
      
    } catch (ConfigParameterException ex) {
      throw new ProcessException(ex);
    }
  }
  
  public void processRecord(ProcessRecord prec) throws ProcessException {
    
    try {
      
      if (logger.LEVEL2) {
        logger.printres("*PROCESSING_RECORD", ""+prec.getIndex() );
      }
      
      String SourceAreas = sourceAreas.resolve(request).stringValue();
      String TargetAreas = targetAreas.resolve(request).stringValue();
      String ResponseHandler = responseHandler.resolve(request).stringValue("AjaxMgr.refreshAreasResponseHandler");
      
      if (logger.LEVEL2) {
        if (sourceAreas.hasMScript() || targetAreas.hasMScript() || responseHandler.hasMScript()) {
          logger.printres("*PROCESS_PARAMETERS_LINE_RESOLVED");
          logger.printres("REFRESH_AREAS_PROCESS_PARAMETERS", SourceAreas, TargetAreas, ResponseHandler );
        }
      }
      
      AjaxResponse response = new AjaxResponse();
      response.setName(XString.afterLast(callDef.getTargetName(),'.') + "." + prec.getIndex());
      
      response.addReponseProperty("targetAreas",TargetAreas);
      
      RefreshAreas.extractSourceAreas(SourceAreas,request,response);
      response.setResponseHandler(ResponseHandler);
      
      AjaxProcessManager.addResponse(request,response);
      if (logger.LEVEL2) logger.printres("RESPONSE_CREATED");
      
    } catch (ConfigurationException ex) {
      if (logger.LEVEL1) logger.printres("*EXCEPTION_OCCURED", ex.getMessage());
      throw new ProcessException(ex);
      
    } catch (AjaxException ex) {
      if (logger.LEVEL1) logger.printres("*EXCEPTION_OCCURED", ex.getMessage());
      throw new ProcessException(ex);
    }
    
  }
  
  public void finalizeProcess() throws ProcessException {
    
    if (logger.LEVEL2) {
      logger.printres("*PROCESS_FINALIZING",callDef.getStepName());
    }
    
  }
  
  
}
