/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * RefreshBlocksProcess.java
 *
 * Created on 05 September 2007 Wednesday, 23:10
 *
 */

package org.moremotion.mmcomp.ajax;

import org.moremotion.adom.ADOM;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.process.Process;
import org.moremotion.process.ProcessException;
import org.moremotion.process.ProcessRecord;
import org.moremotion.util.XString;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: RefreshBlocksProcess.java 199 2008-09-08 08:48:37Z erkan $
 */
public class RefreshBlocksProcess extends Process {
  
  private ConfigParameter dataSources;
  private ConfigParameter nodesToEncrypt;
  private ConfigParameter targetBlocks;
  private ConfigParameter responseHandler;
  private ConfigParameter fieldMap;
  
  public void prepare() throws ProcessException {
    
    if (logger.LEVEL1) {
      logger.printres("*PROCESS_STARTED", callDef.getStepName());
    }
      
    try {
      
      dataSources = processConfig.getParameter("dataSources");
      nodesToEncrypt = processConfig.getParameter("nodesToEncrypt");
      targetBlocks = processConfig.getParameter("targetBlocks");
      fieldMap = processConfig.getParameter("fieldMap");
      responseHandler = processConfig.getParameter("responseHandler");
      
      if (logger.LEVEL2) {
        logger.printres("*PROCESS_PARAMETERS_LINE");
        logger.printres("REFRESH_BLOCKS_PROCESS_PARAMETERS",
            new String[] { dataSources+"", nodesToEncrypt+"", targetBlocks+"", responseHandler+"" });
      }
      
    } catch (ConfigParameterException ex) {
      throw new ProcessException(ex);
    }
  }
  
  public void processRecord(ProcessRecord prec) throws ProcessException {
    
    try {
      
      if (logger.LEVEL2) {
        logger.printres("*PROCESSING_RECORD", ""+prec.getIndex() );
      }
      
      String DataSources = dataSources.resolve(request).stringValue();
      String NodesToEncrypt = nodesToEncrypt.resolve(request).stringValue(null);
      String TargetBlocks = targetBlocks.resolve(request).stringValue();
      String ResponseHandler = responseHandler.resolve(request).stringValue("AjaxMgr.refreshBlocksResponseHandler");
      
      if (logger.LEVEL2) {
        if (dataSources.hasMScript() || nodesToEncrypt.hasMScript() || targetBlocks.hasMScript() || responseHandler.hasMScript()) {
          logger.printres("*PROCESS_PARAMETERS_LINE_RESOLVED");
          logger.printres("REFRESH_AREAS_PROCESS_PARAMETERS", new String[] {DataSources, NodesToEncrypt, TargetBlocks, ResponseHandler});  
        }
      }
      
      String[] dsList = XString.split(DataSources,',');
      String[] nteList = XString.split(NodesToEncrypt == null ? "" : NodesToEncrypt ,';');
      String[] tbList = XString.split(TargetBlocks,',');
      
      for (int i = 0; i < dsList.length; i++) {
        
        if (logger.LEVEL2) {
          logger.printres("PREPARING_RESPONSE_NUM", ""+i);
        }
        AjaxResponse response = new AjaxResponse();
        response.setName(XString.afterLast(callDef.getTargetName(),'.') + "." + prec.getIndex() + "." + i);
        
        response.addReponseProperty("targetBlock",tbList[i]);
        response.addReponseProperty("fieldMap",fieldMap.resolve(request).stringValue(""));
        
        String nte = i < nteList.length - 1 ? nteList[i] : null;
        
        ADOM adom = Utils.generateADOM(dsList[i],request,nte);
        
        response.setADOM(adom);
        // response.addReponseProperty("blockSeq",i+"");
        response.setResponseHandler(ResponseHandler);
        AjaxProcessManager.addResponse(request,response);
        if (logger.LEVEL2) logger.printres("RESPONSE_CREATED");
      }
      
    } catch (ConfigParameterException ex) {
      if (logger.LEVEL1) logger.printres("*EXCEPTION_OCCURED", ex.getMessage());
      throw new ProcessException(ex);
      
    } catch (AjaxException ex) {
      if (logger.LEVEL1) logger.printres("*EXCEPTION_OCCURED", ex.getMessage());
      throw new ProcessException(ex);
    }
    
  }
  
  public void finalizeProcess() throws ProcessException {
    if (logger.LEVEL2) {
      logger.printres("*PROCESS_FINALIZING",callDef.getStepName());
    }
  }
  
  
}
