/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * RefreshOptionsProcess.java
 *
 * Created on 05 September 2007 Wednesday, 23:10
 *
 */

package org.moremotion.mmcomp.ajax;

import org.moremotion.adom.ADOM;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.process.Process;
import org.moremotion.process.ProcessException;
import org.moremotion.process.ProcessRecord;
import org.moremotion.util.XString;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: RefreshOptionsProcess.java 51 2008-03-10 11:46:54Z erkan $
 */
public class RefreshOptionsProcess extends Process {
  
  private ConfigParameter dataSource;
  private ConfigParameter optionNode;
  private ConfigParameter valueNode;
  private ConfigParameter encryptTheValue;
  private ConfigParameter targetElement;
  private ConfigParameter staticOptionsQty;
  private ConfigParameter responseHandler;
  
  public void prepare() throws ProcessException {
    
    if (logger.LEVEL1) {
      logger.printres("*PROCESS_STARTED", callDef.getStepName());
    }
    
    try {
      
      dataSource = processConfig.getParameter("dataSource");
      optionNode = processConfig.getParameter("optionNode");
      valueNode = processConfig.getParameter("valueNode");
      encryptTheValue = processConfig.getParameter("encryptTheValue");
      targetElement = processConfig.getParameter("targetElement");
      staticOptionsQty = processConfig.getParameter("staticOptionsQty");
      responseHandler = processConfig.getParameter("responseHandler");
      
      if (logger.LEVEL2) {
        logger.printres("*PROCESS_PARAMETERS_LINE");
        logger.printres("REFRESH_OPTIONS_PROCESS_PARAMETERS",
            new String[] {dataSource+"", optionNode+"", valueNode+"", encryptTheValue+"",
                          targetElement+"", staticOptionsQty+"", responseHandler+""});
      }
      
      
    } catch (ConfigParameterException ex) {
      throw new ProcessException(ex);
    }
  }
  
  public void processRecord(ProcessRecord prec) throws ProcessException {
    
    try {
      
      if (logger.LEVEL2) {
        logger.printres("*PROCESSING_RECORD", ""+prec.getIndex() );
      }
      
      AjaxResponse response = new AjaxResponse();
      response.setName(XString.afterLast(callDef.getTargetName(),'.') + "." + prec.getIndex());
      
      String DataSource = dataSource.resolve(request).stringValue();
      String OptionNode = optionNode.resolve(request).stringValue();
      String ValueNode = valueNode.resolve(request).stringValue();
      boolean EncryptTheValue = encryptTheValue.resolve(request).booleanValue(false);
      String nodesToEncrypt = EncryptTheValue ? ValueNode : null;
      String TargetElement = targetElement.resolve(request).stringValue();
      String StaticOptionsQty = staticOptionsQty.resolve(request).stringValue();
      String ResponseHandler = responseHandler.resolve(request).stringValue("AjaxMgr.refreshOptionsResponseHandler");
      
      
      if (logger.LEVEL2) {
        if (dataSource.hasMScript() || optionNode.hasMScript() || valueNode.hasMScript() || encryptTheValue.hasMScript() || 
            targetElement.hasMScript() || staticOptionsQty.hasMScript() || responseHandler.hasMScript()) {
          logger.printres("*PROCESS_PARAMETERS_LINE_RESOLVED");
          logger.printres("REFRESH_OPTIONS_PROCESS_PARAMETERS",
              new String[] {DataSource, OptionNode, ValueNode, EncryptTheValue+"",
                            TargetElement, StaticOptionsQty+"", ResponseHandler});
        }
      }
      
      response.addReponseProperty("valueNode",ValueNode);
      response.addReponseProperty("optionNode",OptionNode);
      response.addReponseProperty("targetElement",TargetElement);
      response.addReponseProperty("staticOptionsQty",StaticOptionsQty);
      
      ADOM adom = Utils.generateADOM(DataSource,request,nodesToEncrypt);
      response.setADOM(adom);
      response.setResponseHandler(ResponseHandler);
      
      AjaxProcessManager.addResponse(request,response);
      if (logger.LEVEL2) logger.printres("RESPONSE_CREATED");
      
    } catch (ConfigParameterException ex) {
      if (logger.LEVEL1) logger.printres("*EXCEPTION_OCCURED", ex.getMessage());
      throw new ProcessException(ex);
      
    } catch (AjaxException ex) {
      if (logger.LEVEL1) logger.printres("*EXCEPTION_OCCURED", ex.getMessage());
      throw new ProcessException(ex);
    }
    
  }
  
  public void finalizeProcess() throws ProcessException {
    if (logger.LEVEL2) {
      logger.printres("*PROCESS_FINALIZING",callDef.getStepName());
    }
  }
  
  
}
