/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * Utils.java
 *
 * Created on 08 May 2006 Monday, 17:01
 */

package org.moremotion.mmcomp.ajax;

import java.util.Iterator;
import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMException;
import org.moremotion.adom.ADOMNode;
import org.moremotion.config.ConfigurationException;
import org.moremotion.datasrc.ADOMBuilder;
import org.moremotion.datasrc.DataSourceConfig;
import org.moremotion.datasrc.DataSourceManager;
import org.moremotion.datasrc.RefreshDataSource;
import org.moremotion.page.PageGenerationException;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.Logger;
import org.moremotion.util.Encryption;
import org.moremotion.util.XString;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: Utils.java 199 2008-09-08 08:48:37Z erkan $
 */
public class Utils {
  
  
  protected static ADOM generateADOM(String dsname, MoreMotionRequest request, String nodesToEncrypt) throws AjaxException {
    
    ADOM adom = new ADOM();
    
    DataSourceConfig dsconfig;
    try {
      dsconfig = DataSourceManager.getDataSourceConfig(request, dsname);
    } catch (ConfigurationException ex) {
      throw new AjaxException(ex);
    }
    if (dsconfig == null) {
      throw new AjaxException("DATASOURCE_MISSING",new String[]{dsname},null);
    }
    
    try {
      RefreshDataSource.refresh(request);
    } catch (PageGenerationException ex) {
      throw new AjaxException(ex);
    }
    
    ADOMBuilder builder = new ADOMBuilder(request,dsconfig,new Logger(""));
    
    try {
      adom = builder.build();
    } catch (ADOMException ex) {
      throw new AjaxException(ex);
    }
    
    if (adom != null && nodesToEncrypt != null) {
      encryptNodes(adom, nodesToEncrypt);
    }
    
    return adom;
    
  }
  
  private static void encryptNodes(ADOM adom, String nodesToEncrypt) {
    String nte[] = XString.split(nodesToEncrypt,',');
    _encryptNodes(adom.getRootNode(),nte);
    
  }
  
  private static void _encryptNodes(ADOMNode node, String[] nte) {
    if (node.getNodeCount() > 0) {
      Iterator itr = node.getNodes();
      while (itr.hasNext()) {
        _encryptNodes((ADOMNode)itr.next(),nte);
      }
    } else {
      String nodeName = node.getName();
      for (int i = 0; i < nte.length; i++) {
        if (nodeName.equals(nte[i])) {
          String nodeValue = node.getValue();
          String encValue = Encryption.encstr(nodeValue, nodeName);
          node.setValue(encValue);
        }
      }
    }
  }
  
}
