/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.mmcomp.filer.upload;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import javax.servlet.http.HttpServletRequest;
import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMManager;
import org.moremotion.resource.Resource;

/**
 * @author Mehmet Tastan
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: UploadManager.java 51 2008-03-10 11:46:54Z erkan $
 */

public class UploadManager {
  
  private File webapplRoot;
  private File workingPath;
  private File file;
  private HttpServletRequest request = null;
  private FileOutputStream fos = null;
  
  private ADOM _Message(String message, String[] args, boolean error) {
    
    ADOM adom_sfu =  ADOMManager.newSessionADOM(request.getSession(), "uploadsf_msg");
    adom_sfu.setResBundleName("mor_filer");
    
    Resource res = new Resource(message, args, "mor_filer");
    adom_sfu.setNodeValue("text", res.toString());
    
    if (error) adom_sfu.setNodeValue("error", "true");
    
    return adom_sfu;
  }
  
  public void SuccessMessage(String message, String arg1, String sfn) {
    ADOM adom = _Message(message, new String[]{arg1}, false);
    adom.setNodeValue("filename", sfn);
  }
  
  public void Message(String message) {
    _Message(message, new String[]{}, true);
  }
  
  public void Message(String message, String arg1) {
    _Message(message, new String[]{arg1}, true);
  }
  
  public void Message(String message, String arg1, String arg2) {
    _Message(message, new String[]{arg1, arg2}, true);
  }
  
  public void Message(String message, String arg1, String arg2, String arg3) {
    _Message(message, new String[]{arg1, arg2, arg3}, true);
  }
  
  public boolean openFile(String filename, boolean overwrite) throws Exception {
    
    file = new File( getWorkingPath(), filename);
    
    if (file.exists()) {
      if (!overwrite) {
        Message("UPLOADER:FILE_ALREADY_EXIST",file.getName());
        //          Message("File \"" + file.getName() + "\" already exists.");
        return false;
      }
    }
    
    file.createNewFile();
    this.fos = new FileOutputStream(file);
    return true;
    
  }
  
  public void writeFile(byte[] data, int len) throws IOException {
    this.fos.write(data,0,len);
  }
   
  public void eraseFile() throws IOException {
    fos.flush();
    fos.close();
    file.delete(); 
  }
  
  public void closeFile(String serverFileName) throws IOException {
    fos.flush();
    fos.close();
    fos = null;
    SuccessMessage("UPLOADER:FILE_UPLOADED_SUCCESSFULLY", file.getName(), serverFileName);
  }
  
  
  public UploadManager(HttpServletRequest request, String root) {
    this.request = request;
    webapplRoot = new File(root);
  }
  
  private boolean absoluteDefinition(String name) {
    return
      name.startsWith("/") ||
      name.startsWith("\\") ||
      name.indexOf(':') > -1;
  }
  
  public void setWorkingPath(String path) {
    
    if (path.length() == 0) {
      workingPath = webapplRoot;
    } else if (absoluteDefinition(path)) {
      workingPath = new File(path);
    } else {
      workingPath = new File(webapplRoot, path);
    }
    
    workingPath.mkdirs();
  }
  
  public File getWorkingPath() {
    return this.workingPath;
  }
   
}