/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.mmcomp.reldb;

import java.sql.CallableStatement;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.NumberFormat;
import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMException;
import org.moremotion.adom.ADOMNode;
import org.moremotion.cache.VariablePool;
import org.moremotion.config.Config;
import org.moremotion.config.ConfigManager;
import org.moremotion.config.ConfigParameter;
import org.moremotion.datasrc.DataSourceConfig;
import org.moremotion.datasrc.Paging;
import org.moremotion.evaluator.MMSymbolResolver;
import org.moremotion.evaluator.MScriptResolver;
import org.moremotion.exception.ExceptionUtils;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.Logger;
import org.moremotion.util.JdbcConnection;
import org.moremotion.util.JdbcUtils;
import org.moremotion.util.XString;

/**
 * Queries a Relational Database using the given DataSourceConfig and Paging ojects
 * and fills in the given {@link org.moremotion.adom.ADOM ADOM} with the result records
 *
 * @author Yusuf Boyacigil
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ReldbQuery.java 215 2008-09-13 10:27:24Z erkan $
 */

public class ReldbQuery {
  
  private MoreMotionRequest request;
  private Paging paging;
  private DataSourceConfig dsconfig;
  private Logger logger;
  private int noi = 0;
  private BlobExportDefinition bed = null;
  
  
  /**
   * Creates a new instance of ReldbQuery
   */
  public ReldbQuery(MoreMotionRequest request, DataSourceConfig dsconfig, Paging paging, Logger logger) {
    this.request = request;
    this.dsconfig = dsconfig;
    this.paging = paging;
    this.logger = logger;
  }


  public void execute(ADOM adom) throws ADOMException {
    
    noi = 0;
    bed = null;
    
    if (logger.LEVEL1) {
      logger.printres("DEBUGGING_DATASOURCE_STARTED", dsconfig.getName());
    }
    
    String querypart = null;
    
    Connection con = null;
    Statement stmt = null;
    CallableStatement cstmt = null;
    ResultSet rs = null;
    ResultSet rsSpecial = null;
    
    try {
      
      String conName = dsconfig.getParameter("conn").resolve(request).stringValue();
      
      Config conConfig = ConfigManager.getConfig("jdbcConnection",conName);
      if (conConfig == null) {
        throw new ReldbADOMException("CONNECTION_CONFIG_MISSING",new String[]{conName},null);  
      }
      
      String Dburl = conConfig.getParameter("dburl").resolve(request).stringValue("");
      
      int RefreshTimeOut = dsconfig.getRefreshTimeout(request);
      
      boolean keepResultSet = false;
      boolean pagingNeeded = dsconfig.getItemsPerPage(request) > 0;
      boolean navigation = false;
      
      String qs = dsconfig.getParameter("query").stringValue();
      
      if (pagingNeeded) {
        String rdsr = (String)request.getAttribute("moremotion.refresh.datasource." + dsconfig.getName());
        navigation = rdsr != null && rdsr.equals("true");
        
        if (RefreshTimeOut > 0) {
          keepResultSet = true;
        }
      }
      
      String key = null;
      if (keepResultSet) {
        key = request.getSession().getId() + "_" + dsconfig.getName();
        if (navigation) {
          rs = ResultSetCache.getInstance().getResultSet(key);
          if (rs != null) {
            rs.beforeFirst();
          }
        } else {
          ResultSetCache.getInstance().removeResultSet(key);
        }
      }
      
      boolean SPCall = false;
      querypart = null;
      String savedQueryPart = null;
      
      if (rs == null) {
        
        if (pagingNeeded && keepResultSet) {
          con = JdbcConnection.getConnection(null, conName);
        } else {
          con = JdbcConnection.getConnection(request, conName);
        }
        
        MMSymbolResolver mmsr = request.getSymbolResolver();
//        mmsr.setProcessRecord(null);
        MScriptResolver resolver = new MScriptResolver(qs, mmsr, ';');
        resolver.setSqlInjectionCheck(true);
        
        ConfigParameter driver = dsconfig.getParameter("driver");
        ConfigParameter doubleBS = dsconfig.getParameter("doubleBS");
        
        boolean doublebs = driver != null && driver.resolve(request).stringValue("").indexOf("mysql") > -1;
        doublebs = doublebs || (doubleBS != null && doubleBS.resolve(request).booleanValue(false));
        
        if (doublebs) {
          resolver.setDoubleBackSlashes(true);
        }

        String ignoreChars = 
            dsconfig.getParameter("ignoreChars").resolve(request).stringValue(null);

        boolean ignoreCR = ignoreChars != null && ignoreChars.equalsIgnoreCase("cr");
        boolean ignoreLF = ignoreChars != null && ignoreChars.equalsIgnoreCase("lf");
      
        
        String qpart = resolver.nextStatement();
        while ( qpart != null) {
          
          if (qpart.trim().length() == 0) {
            qpart = resolver.nextStatement();
            continue;
          }
          
          querypart = qpart;
          
          if (logger.LEVEL1) logger.printres("QUERY_PART", querypart);
          
          if (ignoreCR) querypart = XString.replace(querypart,"\r","");
          else if (ignoreLF) querypart = XString.replace(querypart,"\n","");
          
          boolean special;
          if (querypart.startsWith("selecT")) {
            querypart = "select" + querypart.substring(6);
            special = true;
            querypart = JdbcUtils.arrangeSpecialSQL(querypart);
            querypart = XString.replace(querypart,"vpool:","vpool_");
          } else {
            special = false;
            savedQueryPart = new String(querypart);
          }
          
          bed = null;
          if (org.moremotion.mmcomp.reldb.BlobExportDefinition.hasBlobExportDefinition(querypart)) {
            if (logger.LEVEL2) logger.printres("BLOB_EXPORT_DEFINITION_ENCOUNTERED");
            bed = new org.moremotion.mmcomp.reldb.BlobExportDefinition(querypart, Dburl);
          }
          
          SPCall = hasStoredProcedureCall(querypart);
          
          if (SPCall) {
            if (logger.LEVEL2) logger.printres("STORED_PROCEDURE_CALL_REQUESTED");
            if (pagingNeeded && keepResultSet && !special) {
              cstmt = con.prepareCall(querypart,ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
            } else {
              cstmt = con.prepareCall(querypart);
            }
            rs = cstmt.executeQuery();
            
          } else {
            if (pagingNeeded && keepResultSet && !special) {
              stmt = con.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
            } else {
              stmt = con.createStatement();
            }
            
            long started = 0;
            if (logger.LEVEL1) {
              logger.printres("BEFORE_EXECUTE_QUERY", printUsedMem());
              started = System.currentTimeMillis();
            }
            
            if (special) {
              rsSpecial = stmt.executeQuery(querypart);
            } else {
              rs = stmt.executeQuery(querypart);
            }
            
            if (logger.LEVEL1) {
              long finished = System.currentTimeMillis();
              logger.printres("AFTER_EXECUTE_QUERY", printUsedMem(), ""+(finished - started));
            }
            
            if (keepResultSet && !special) {
              ResultSetCache.getInstance().addResultSet(key,rs,RefreshTimeOut);
            }
          }
          
          if (special) {
            setFields(rsSpecial, logger);
            rsSpecial.close();
            querypart = null;
          }
          
          qpart = resolver.nextStatement();
        }
        
        resolver.setSqlInjectionCheck(false);
        
      }
      
      if (rs == null) {
        return;
      }
      
      /*
       * Paging support required ?
       */
      if (!pagingNeeded) {
        
        fillADOM(adom, rs,logger);
        
      } else {
        
        /*
         * Yes, Paging support required.
         */
        
        noi = 0;
        if (!skip(rs, paging.getSkipAmount())) {
          /*
           * Desired skip not successful. Re-Execute the query
           */
          
          paging.setNumberOfItems(noi);
          noi = 0;
          
          if (keepResultSet) {
            rs.beforeFirst();
          } else {
            if (SPCall) {
              rs = cstmt.executeQuery();
            } else {
              rs = stmt.executeQuery(savedQueryPart);
            }
          }
          skip(rs, paging.getSkipAmount());
        }
        
        fillADOM(adom, rs, paging.getItemsPerPage(), paging.getPagesPerBlock(), logger);
        
        if (paging.getSkipAmount() > 0 && adom.getNodeCount() == 0) {
          
          paging.setNumberOfItems(noi);
          noi = 0;
          
          if (keepResultSet) {
            /*
             * Result set scrollable. No need to re-execute
             */
            rs.beforeFirst();
            
          } else {
            if (SPCall) {
              rs = cstmt.executeQuery();
            } else {
              rs = stmt.executeQuery(querypart);
            }
          }
          skip(rs, paging.getSkipAmount());
          fillADOM(adom, rs, paging.getItemsPerPage(), paging.getPagesPerBlock(), logger);
        }
        
        paging.setNumberOfItems(noi);
        // paging.addPagingValues(adom);
        
      }
      
//      if (paging.getTotalNumberOfItems() != -1) {
//        result.setNodeValue("total-number-of-items",paging.getTotalNumberOfItems() + "");
//     }
      
      if (pagingNeeded && keepResultSet) {
      } else {
        if (stmt != null) stmt.close();
        else if (cstmt != null) cstmt.close();
        // con.close();
      }
      
    } catch(Exception e) {
      
      try {
        if (rs != null) rs.close();
        if (rsSpecial != null) rsSpecial.close();
        if (stmt != null) stmt.close();
        if (cstmt != null) cstmt.close();
        // if (con != null) con.close();
      } catch (Exception ex) {}
      
      logger.printres("SQL_EXECUTION_ERROR_IN_DATASOURCE",dsconfig.getName(), ExceptionUtils.getStackTrace(e),querypart); 

      throw new ADOMException(e);
    }
    
    return;    
    
    
  }
  
  private int getCount(ResultSet rs) {
    int c = 0;
    try {
      if(rs.last()) {
        c = rs.getRow();
      }
      rs.beforeFirst();
    } catch (Exception e){
      e.printStackTrace();
    }
    return c;
  }
  
  private boolean skip(ResultSet rs, int count) throws SQLException {
    for (int i = 0; i < count; i++) {
      if (!rs.next()) {
        paging.setTotalNumberOfItems(noi);
        return false;
      }
      noi++;
    }
    return true;
  }
  
  private String printUsedMem() {
    NumberFormat format = NumberFormat.getInstance();
    Runtime runTime = Runtime.getRuntime();
    return format.format(runTime.totalMemory() - runTime.freeMemory());
  }
  
  private boolean hasStoredProcedureCall(String query) {
    return query.trim().startsWith("{") && query.trim().endsWith("}");
  }
  
  
  private void fillADOM(ADOM result, ResultSet rs, Logger logger) throws Exception {
    
    FastMetaData fmd = new FastMetaData(rs);
    int ctrx = 0;
    
    while(rs.next()) {
      
      ADOMNode rowNode = new ADOMNode("item");
      
      ctrx++;
      
      if (logger.LEVEL3) logger.print(ctrx + ". Record ...");
      
      for(int i = 1; i <= fmd.columnCount(); i++) {
        String colName = fmd.name(i);
        String colValue;
        
        if (bed != null && bed.isDefinedAsBlob(colName)) {
          colValue = BlobExporter.exportBlob(request,rs,i,bed);
        } else {
          if (fmd.goodForXml(i)) {
            
            if (fmd.isClob(i)) {
              Clob clob = rs.getClob(i);
              if (clob == null) colValue = "";
              else {
                long pos = 1;
                Long len = new Long(clob.length());
                colValue = clob.getSubString(pos,len.intValue());
              }
            } else {
              colValue = rs.getString(i);
            }
            
          } else {
            colValue = "{" + fmd.type(i) + "_DATA}";
          }
        }
        
        rowNode.setNodeValue(colName,colValue);
        
        if (logger.LEVEL3) logger.print("  " + fixedWidth(colName) + " : " + safe(colValue));
      }
      result.addNode(rowNode);
      
    }
    paging.setTotalNumberOfItems(ctrx);
    
  }
  
  private void fillADOM(ADOM result, ResultSet rs, int amount, int ppb, Logger logger) throws Exception {
    
    FastMetaData fmd = new FastMetaData(rs);
    int count = 0;
    boolean eof = true;
    
    while(rs.next()) {
      
      count++;
      noi++;
      
      if (count <= amount) {
          
        if (logger.LEVEL3) logger.print(count + ". Record ...");
      
        ADOMNode rowNode = new ADOMNode("item");
        for(int i = 1; i <= fmd.columnCount(); i++) {
          String colName = fmd.name(i);
          String colValue;
          if (bed != null && bed.isDefinedAsBlob(colName)) {
            colValue = BlobExporter.exportBlob(request,rs,i,bed);
          } else {
            if (fmd.goodForXml(i)) {
              
              if (fmd.isClob(i)) {
                Clob clob = rs.getClob(i);
                if (clob == null) colValue = "";
                else {
                  long pos = 1;
                  Long len = new Long(clob.length());
                  colValue = clob.getSubString(pos,len.intValue());
                }
              } else {
                colValue = rs.getString(i);
              }
              
            } else {
              colValue = "{" + fmd.type(i) + "_DATA}";
            }
          }
          
          rowNode.setNodeValue(colName,colValue);
          
          if (logger.LEVEL3) logger.print("  " + fixedWidth(colName) + ":" + safe(colValue));
        }
        
        result.addNode(rowNode);
      } else {
        if (ppb < 1) {
          eof = false;
          boolean hasMore = rs.next();
          paging.setMorePages(hasMore || noi > paging.getItemsPerPage());
          paging.setLastPage(!hasMore);
          break;
        }
      }
    }
    if (eof) {
      paging.setTotalNumberOfItems(noi);
      paging.setMorePages(noi > paging.getItemsPerPage());
      paging.setLastPage(count <= amount);
    }
    
  }
  
  
  
  private String fixedWidth(String s) {
    return (s + "                   ").substring(0,20);
  }
  
  private String safe(String s) {
    return s == null ? "" : s;
  }
  
  private void setFields(ResultSet rs, Logger logger) throws Exception {
    
    ResultSetMetaData rsmd = rs.getMetaData();
    boolean hasResult = rs.next();
    
    int cc = rsmd.getColumnCount();
    
    VariablePool vpool = request.getVPool();
    for (int ci = 1; ci <= cc; ci++) {
      
      String colName = rsmd.getColumnName(ci);
      String colValue = "";
      if (hasResult) {
        colValue = rs.getString(ci);
        if (colValue == null) {
          colValue = "";
        }
      }
      if (colName.toLowerCase().startsWith("vpool_")) {
        colName = colName.substring(6);
        vpool.put(colName,colValue);
        if (logger.LEVEL1) logger.print("Pool variable '" + colName + "' set to '" + colValue + "'");
        
      } else if (colName.toLowerCase().startsWith("ppool_")) {
        colName = colName.substring(6);
        request.getVars().setPageVar(colName, colValue);
        if (logger.LEVEL1) logger.print("Page variable '" + colName + "' set to '" + colValue + "'");
        
      } else if (colName.toLowerCase().startsWith("spool_")) {
        colName = colName.substring(6);
        request.getVars().setSessionVar(colName, colValue);
        if (logger.LEVEL1) logger.print("Session variable '" + colName + "' set to '" + colValue + "'");
      }
      
    }
  }
    
  
  
}
