/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.mmcomp.reldb;

import java.sql.ResultSet;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ResultSetCache.java 215 2008-09-13 10:27:24Z erkan $
 */
public class ResultSetCache implements Runnable {
  
  private Thread t;
  private boolean stop = false;
  
  private static HashMap cache = new HashMap();
  
  private static ResultSetCache Instance;
  
  private ResultSetCache() {
    t = new Thread(this);
    t.setDaemon(true);
    t.start();
  }
  
  /**
   * Returns a single instance of ResultSetCache class.
   */
  public static synchronized ResultSetCache getInstance() {
    if (Instance == null) {
      synchronized (Instance) {
        if (Instance == null) {
          Instance = new ResultSetCache();
        }
      }
    }
    return Instance;
  }
  
  public void addResultSet(String key, ResultSet rs, long refreshTimeout) {
    
    synchronized (cache) {
      RSCacheEntry ce = new RSCacheEntry();
      ce.refreshTimeout = refreshTimeout;
      ce.lastUsed = (new java.util.Date()).getTime();
      ce.rs = rs;
      cache.put(key, ce);
    }
    
  }
  
  public ResultSet getResultSet(String key) {
    
    RSCacheEntry ce = (RSCacheEntry)cache.get(key);
    if (ce == null) {
      return null;
    }
    ce.lastUsed = (new java.util.Date()).getTime();
    return ce.rs;
  }
  
  public void removeResultSet(String key) {
    
    RSCacheEntry ce = (RSCacheEntry) cache.get(key);
    
    try {
      Statement stmt = ce.rs.getStatement();
      if (stmt != null) {
        java.sql.Connection con = stmt.getConnection();
        stmt.close();
        con.close();
      }
    } catch (Exception e) {}
    
    
    synchronized (cache) {
      cache.remove(key);
    }
    
  }
  
  public void run() {
    
    while(!stop) {
      
      try {
        
        synchronized (cache) {
          
          long curr = (new java.util.Date()).getTime();
          
          Iterator itr = cache.entrySet().iterator();
          
          while(itr.hasNext()) {
            
            Map.Entry entry = (Map.Entry)itr.next();
            String key = (String)entry.getKey();
            RSCacheEntry ce = (RSCacheEntry) entry.getValue();
            
            if (curr > ce.lastUsed + (ce.refreshTimeout * 1000L)) {
              
              Statement stmt = ce.rs.getStatement();
              if (stmt != null) {
                java.sql.Connection con = stmt.getConnection();
                stmt.close();
                con.close();
              }
              itr.remove();
            }
          }
        }
        
        t.sleep(1000L * 10 * 1);  // Every 10 seconds
        
      } catch(Exception ex) {
        ex.printStackTrace();
      }
      
    }
  }
  
  public void stop() {
    this.stop = true;
  }
  
  public void finalize() {
    stop();
  }
  
}

class RSCacheEntry {
  protected long refreshTimeout;
  protected long lastUsed;
  protected ResultSet rs;
  public RSCacheEntry() {}
}
