/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * UpdateBLOBFieldProcess.java
 *
 * Created on 17 March 2006 Friday, 12:06
 */

package org.moremotion.mmcomp.reldb;

import java.sql.Clob;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.evaluator.EvaluationException;
import org.moremotion.evaluator.Expression;
import org.moremotion.evaluator.MMSymbolResolver;
import org.moremotion.process.ProcessException;
import org.moremotion.process.Process;
import org.moremotion.process.ProcessRecord;
import org.moremotion.resource.Resource;
import org.moremotion.evaluator.MScriptResolver;
import org.moremotion.util.JdbcConnection;
import org.moremotion.util.JdbcConnectionException;
import org.moremotion.util.StringParser;


/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: UpdateClobFieldProcess.java 215 2008-09-13 10:27:24Z erkan $
 */
public class UpdateClobFieldProcess extends Process {
  
  private ConfigParameter sql;
  private ConfigParameter fields;
  private Connection con;
  
  public void prepare() throws ProcessException {
    
    if (logger.LEVEL1) {
      logger.printres("*PROCESS_STARTED", callDef.getStepName());
    }
    
    try {
      String conName = processConfig.getParameter("conn").resolve(request).stringValue();  
      sql = processConfig.getParameter("query");
      fields = processConfig.getParameter("fields");
      
      if (logger.LEVEL2) {
        logger.printres("*PROCESS_PARAMETERS_LINE");
        logger.printres("UPDATE_CLOB_PROCESS_PARAMETERS", conName.toString(), sql.toString(), fields.toString() );
      }
      
      con = JdbcConnection.getConnection(request, conName);
      if (logger.LEVEL1) {
        logger.printres("JDBC_CONNECTION_ESTABLISHED",new String[]{conName});
      }
      
    } catch (ConfigParameterException ex) {
      throw new ProcessException(ex);
    } catch (JdbcConnectionException ex) {
      throw new ProcessException(ex);
    }
  }
  
  public void processRecord(ProcessRecord prec) throws ProcessException {
    
    try {
      
      if (logger.LEVEL2) {
        logger.printres("*PROCESSING_RECORD", ""+prec.getIndex() );
      }
      
      String Sql = sql.resolve(request).stringValue();
      if (logger.LEVEL1 && sql.hasMScript()) {
        logger.printres("QUERY_PART", Sql );
      }
      
      String Fields = fields.resolve(request).stringValue();
      if (logger.LEVEL1 && fields.hasMScript()) {
        logger.printres("CLOB_ASSIGNMENTS_RESOLVED", Fields );
      }
      
      processRecord(prec, con, Sql, Fields);
      
    } catch (ConfigParameterException ex) {
      throw new ProcessException(ex);
    }
    
  }
  
  private String normalize(String str, char dlm) {
    StringBuffer buf = new StringBuffer(str.length()+2);
    StringParser parser = new StringParser(str,dlm);
    while (parser.hasMore()) {
      if (buf.length() > 0) buf.append("\n");
      buf.append(parser.nextItem());
    }
    return buf.toString();
  }
  
  private void processRecord(ProcessRecord prec, Connection con, String sql, String fields) throws ProcessException {
    
    try {
      
      Statement stmt = con.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE);
      
      MMSymbolResolver mmsr = request.getSymbolResolver();
      
      HashMap h = new HashMap();
      
      fields = normalize(fields,',');
      fields = normalize(fields,';');
      
      StringTokenizer st = new StringTokenizer(fields,"\n");
      while (st.hasMoreTokens()) {
        
        String line = st.nextToken().trim();
        if (line.length() == 0)
          continue;
        
        String fn = null;
        String fv = null;
        
        int pos = line.indexOf('=');
        if (pos == -1) {
          String details = "" + new Resource("INVALID_FIELD_VALUE_DETAILS", new String[]{fields});
          createMessage("INVALID_FIELD_ASSIGNMENT",new String []{line}, details);
          setCompletionCode(12);
          if (logger.LEVEL1) {
            logger.printres("INVALID_FIELD_ASSIGNMENT_LOG");
          }
          return;
          
        } else {
          fn = line.substring(0,pos).trim();
          String sn = line.substring(pos+1).trim();
          
          if (!MScriptResolver.isSymbol(fn)) {
            String details = "" + new Resource("INVALID_FIELD_VALUE_DETAILS", new String[]{fields});
            createMessage("INVALID_FIELD_NAME",new String []{fn, line}, details);
            setCompletionCode(12);
            if (logger.LEVEL1) {
              logger.printres("INVALID_FIELD_NAME_LOG");
            }
            return;
            
       /* } else if (!MScriptResolver.isSymbol(sn)) {
            String details = "" + new Resource("INVALID_FIELD_VALUE_DETAILS", new String[]{fields});
            createMessage("INVALID_FIELD_NAME",new String []{sn, line}, details);
            setCompletionCode(12);
            if (logger.LEVEL1) {
              logger.printres("INVALID_FIELD_NAME_LOG");
            }
            return;
       */     
          } else {
            fv = (new Expression(sn)).eval(mmsr);
          }
        }
        
        if (logger.LEVEL3) {
          logger.print(fixedWidth(fn) + " : " + fv );
        }
        
        h.put(fn, fv);
      }
      
      ResultSet rs = stmt.executeQuery(sql);
      while (rs.next()) {
        
        Iterator itr = h.entrySet().iterator();
        while (itr.hasNext()) {
          Map.Entry entry = (Map.Entry)itr.next();
          String fn = (String)entry.getKey();
          String fv = (String)entry.getValue();
          Clob c = rs.getClob(fn);
          
          if (c == null) {
            createMessage("CLOB_COLUMN_IS_NULL",new String []{fn}, null); 
            setCompletionCode(12);
            return;
          }  
          
          c.truncate(0);
          c.setString(1, fv);
          rs.updateClob(fn,c);
          
          if (logger.LEVEL3) {
            logger.printres("CLOB_FIELD_IS_SET", fn, fv.length() < 31 ? fv : fv.substring(0,30) + " ...");
          }
          
        }
        rs.updateRow();
        
      }
      
      rs.close();
      stmt.close();
      
    } catch (EvaluationException ex) {
      throw new ProcessException(ex);
    } catch (SQLException ex) {
      throw new ProcessException(ex);
    }
  }
  
  
  private String fixedWidth(String s) {
    return (s + "                   ").substring(0,20);
  }
  
  public void finalizeProcess() throws ProcessException {
  }
  
}
