/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * GetDetails.java
 *
 * Created on 29 August 2004 Sunday, 19:44
 */

package org.moremotion.mmcomp.security;

import java.util.Enumeration;
import java.util.Properties;
import org.moremotion.datasrc.DataService;
import org.moremotion.datasrc.DataServiceContext;
import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMNode;
import org.moremotion.config.Config;
import org.moremotion.evaluator.MMSymbolResolver;
import org.moremotion.security.CheckPoint;
import org.moremotion.security.Role;
import org.moremotion.security.SecurityContext;
import org.moremotion.security.User;
import org.moremotion.servlet.ServiceException;
import org.moremotion.servlet.MoreMotionRequest;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: GetDetails.java 51 2008-03-10 11:46:54Z erkan $
 */
public class GetDetails implements DataService {
  
  public void doService(ADOM adom, DataServiceContext dsc) throws ServiceException {
    
    adom.clear();
    
    MoreMotionRequest request = dsc.getRequest();
    
    try {
      
      String secdom = Utils.getSecurityDomainName(request, dsc.getDataSourceConfig());
      Config sdconfig = Utils.getSecurityDomainConfig(secdom);
      SecurityContext sctx = Utils.getSecurityContext(request, sdconfig);
      
      Enumeration en = request.getParameterNames();
      while (en.hasMoreElements()) {
        String n = (String)en.nextElement();
        if (!n.equals("pg") && !n.startsWith("_")) {
          sctx.setVar(n,request.getParameter(n,""));
        }
      }
      MMSymbolResolver mmsr = request.getSymbolResolver();
      
      String username = mmsr.getValue("username");
      if (username != null && username.length() > 0) {
        try {
          User user = sctx.getUser(username);
          setUserDetailsToADOM(adom,user);
        } catch (Exception e) {}
      }
      
      String rolename = mmsr.getValue("rolename");
      if (rolename != null && rolename.length() > 0) {
        
        try {
          Role role = sctx.getRole(rolename);
          setRoleDetailsToADOM(adom,role);
        } catch (Exception e) {}
      }
      
      String cpname = mmsr.getValue("cpname");
      if (cpname != null && cpname.length() > 0) {
        try {
          CheckPoint cp = sctx.getCheckPoint(cpname);
          setCheckPointDetailsToADOM(adom,cp);
        } catch (Exception e) {}
      }
      
    } catch (Exception e) {
      request.createMessage(e);
      request.generateDefaultErrorPage();
    }
    
  }
  
  private static void setUserDetailsToADOM(ADOM uinfo, User user) {
    
    uinfo.setNodeValue("id", "" + user.getId());
    uinfo.setNodeValue("username", user.getName());
    uinfo.setNodeValue("email", user.getEmail());
    uinfo.setNodeValue("fullname", user.getFullname());

    uinfo.setNodeValue("disabled", ""+user.isAccountDisabled());
    
    Properties userAttributes = user.getAttributes();
    if (userAttributes != null) {
      Enumeration e = userAttributes.keys();
      while (e.hasMoreElements()) {
        String name = (String)e.nextElement();
        String value = userAttributes.getProperty(name);
        uinfo.setNodeValue(name, value);
      }
    }
    
    Role[] roles = user.getRoles();
    if (roles != null) {
      ADOMNode rolesNode = new ADOMNode("roles","");
      uinfo.addNode(rolesNode);
      for (int i = 0; i < roles.length; i++) {
        ADOMNode node = new ADOMNode("role",roles[i].getName());
        rolesNode.addNode(node);
      }
    }
    
  }
  
  private static void setRoleDetailsToADOM(ADOM adom, Role role) {
    adom.setNodeValue("name",role.getName());
    adom.setNodeValue("desc",role.getDescription());
    
    Properties attrs = role.getAttributes();
    if (attrs != null) {
      Enumeration en = attrs.propertyNames();
      while (en.hasMoreElements()) {
        String attrName = (String)en.nextElement();
        String attrValue = safeval(attrs.getProperty(attrName));
        adom.setNodeValue(attrName, attrValue);
      }
    }
    
    CheckPoint[] cps = role.getCheckPoints();
    if (cps != null) {
      ADOMNode cpsnode = new ADOMNode("checkpoints","");
      adom.addNode(cpsnode);
      for (int i = 0; i < cps.length; i++) {
        ADOMNode node = new ADOMNode("checkpoint",cps[i].getName());
        cpsnode.addNode(node);
      }
    }
    
  }
  
  private static void setCheckPointDetailsToADOM(ADOM adom, CheckPoint cp) {
    adom.setNodeValue("name",cp.getName());
    adom.setNodeValue("desc",cp.getDescription());
    Properties attrs = cp.getAttributes();
    if (attrs != null) {
      Enumeration en = attrs.propertyNames();
      while (en.hasMoreElements()) {
        String attrName = (String)en.nextElement();
        String attrValue = safeval(attrs.getProperty(attrName));
        adom.setNodeValue(attrName, attrValue);
      }
    }
  }
  
  
  private static String safeval(String s) {
    if (s == null) return "";
    return s;
  }

  
}
