/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * SML.java
 *
 * Created on 06 August 2004 Friday, 17:31
 */

package org.moremotion.mmcomp.security;

import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMNode;
import org.moremotion.clang.CLException;
import org.moremotion.clang.CustomLanguage;
import org.moremotion.clang.FunctionSyntax;
import org.moremotion.clang.LanguageSyntax;
import org.moremotion.clang.ParameterSyntax;
import org.moremotion.config.Config;
import org.moremotion.evaluator.MMSymbolResolver;
import org.moremotion.evaluator.Symbol;
import org.moremotion.security.CheckPoint;
import org.moremotion.security.Role;
import org.moremotion.security.SecurityContext;
import org.moremotion.security.User;
import org.moremotion.security.MissingSecurityEntityException;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.Logger;
import org.moremotion.util.StringParser;
import org.moremotion.util.XString;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: SML.java 72 2008-03-18 14:12:35Z erkan $
 */
public class SML extends CustomLanguage {
  
  private SecurityContext sctx;
  private Config sdconfig = null;
  
  private final static int __LoginUser = 1;
  private final static int __LogoutUser = 2;
  private final static int __CreateUser = 3;
  private final static int __UpdateUser = 4;
  private final static int __RemoveUser = 5;
  private final static int __ChangePassword = 6;
  private final static int __ResetPassword = 7;
  private final static int __AddRoleToUser = 8;
  private final static int __RemoveRoleFromUser = 9;
  private final static int __SetRole = 10;
  private final static int __RemoveRole = 11;
  private final static int __AddCheckPointToRole = 12;
  private final static int __RemoveCheckPointFromRole = 13;
  private final static int __SetCheckPoint = 14;
  private final static int __RemoveCheckPoint = 15;
  private final static int __SetVar = 16;
  private final static int __HasCheckPoint = 17;
  private final static int __HasUserAuthority = 18;
  private final static int __SetSubdomain = 19;
  private final static int __GetAccessUrl = 20; 
  private final static int __SendResetPasswordMail = 21; 
  
  
  // private final static int __TestFunction = 99;
  
  private static LanguageSyntax lang;
  private String secdom;
  
  static {
    
    lang = new LanguageSyntax("SML", new FunctionSyntax[] {
      new FunctionSyntax(__LoginUser, "LoginUser", new ParameterSyntax[] {
        new ParameterSyntax(ParameterSyntax.PT_STRING, "User Name", true),
        new ParameterSyntax(ParameterSyntax.PT_PASSWORD, "Password",  false),
        new ParameterSyntax(ParameterSyntax.PT_BOOLEAN, "Multiple Login",  false)
      }
      ),
      new FunctionSyntax(__LogoutUser, "LogoutUser", new ParameterSyntax[] {
        new ParameterSyntax(ParameterSyntax.PT_STRING, "User Name", false),
      }
      ),
      new FunctionSyntax(__CreateUser, "CreateUser", new ParameterSyntax[] {
        new ParameterSyntax(ParameterSyntax.PT_STRING, "User Name",  true),
        new ParameterSyntax(ParameterSyntax.PT_PASSWORD, "Password",  true),
        new ParameterSyntax(ParameterSyntax.PT_STRING, "Full Name",  false),
        new ParameterSyntax(ParameterSyntax.PT_STRING, "E-Posta",  false),
        new ParameterSyntax(ParameterSyntax.PT_BOOLEAN, "Disabled",  false),
        new ParameterSyntax(ParameterSyntax.PT_STRING, "Attributes",  false),
        new ParameterSyntax(ParameterSyntax.PT_STRING, "Pool Variable",  false)
      }
      ),
      new FunctionSyntax(__UpdateUser, "UpdateUser", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_STRING, "User Name", true),
        new ParameterSyntax(ParameterSyntax.PT_STRING, "Full Name",  false),
        new ParameterSyntax(ParameterSyntax.PT_STRING, "E-Posta",  false),
        new ParameterSyntax(ParameterSyntax.PT_BOOLEAN, "Disabled",  false),
        new ParameterSyntax(ParameterSyntax.PT_STRING, "Attributes",  false)
      }
      ),
      new FunctionSyntax(__RemoveUser, "RemoveUser", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_STRING, "User Name", true)
      }
      ),
      new FunctionSyntax(__HasUserAuthority, "HasUserAuthority", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_STRING, "User Name", true),
        new ParameterSyntax( ParameterSyntax.PT_STRING, "CheckPoint", true),
        new ParameterSyntax( ParameterSyntax.PT_SYMBOL, "Symbol", true)
      }
      ),
      new FunctionSyntax(__ChangePassword, "ChangePassword", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_PASSWORD, "Old Password", true),
        new ParameterSyntax( ParameterSyntax.PT_PASSWORD, "New Password", true),
        new ParameterSyntax( ParameterSyntax.PT_PASSWORD, "New Pass. Again", true)
      }
      ),
      new FunctionSyntax(__ResetPassword, "ResetPassword", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_STRING, "User Name", true),
        new ParameterSyntax( ParameterSyntax.PT_PASSWORD, "New Password", true),
        new ParameterSyntax( ParameterSyntax.PT_PASSWORD, "New Pass. Again", true)
      }
      ),
      new FunctionSyntax(__AddRoleToUser, "AddRoleToUser", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_STRING, "User Name", true),
        new ParameterSyntax( ParameterSyntax.PT_STRING, "Role Name", true)
      }
      ),
      new FunctionSyntax(__RemoveRoleFromUser, "RemoveRoleFromUser", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_STRING, "User Name", true),
        new ParameterSyntax( ParameterSyntax.PT_STRING, "Role Name", true)
      }
      ),
      new FunctionSyntax(__SetRole, "SetRole", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_STRING, "Role Name", true),
        new ParameterSyntax( ParameterSyntax.PT_STRING, "Role Desc.", false)
      }
      ),
      new FunctionSyntax(__RemoveRole, "RemoveRole", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_STRING, "Role Name", true)
      }
      ),
      new FunctionSyntax(__AddCheckPointToRole, "AddCheckPointToRole", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_STRING, "Role Name", true),
        new ParameterSyntax( ParameterSyntax.PT_STRING, "CheckPoint Name", true)
      }
      ),
      new FunctionSyntax(__RemoveCheckPointFromRole, "RemoveCheckPointFromRole", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_STRING, "Role Name", true),
        new ParameterSyntax( ParameterSyntax.PT_STRING, "CheckPoint Name", true)
      }
      ),
      new FunctionSyntax(__SetCheckPoint, "SetCheckPoint", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_STRING, "CheckPoint Name", true),
        new ParameterSyntax( ParameterSyntax.PT_STRING, "CheckPoint Desc", false)
      }
      ),
      new FunctionSyntax(__RemoveCheckPoint, "RemoveCheckPoint", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_STRING, "CheckPoint Name", true)
      }
      ),
      new FunctionSyntax(__SetSubdomain, "SetSubdomain", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_STRING, "SubDomain Name", true)
      }
      ),
      new FunctionSyntax(__SetVar, "SetVar", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_STRING, "Variable Name", true),
        new ParameterSyntax( ParameterSyntax.PT_STRING, "Variable Value", true)
      }
      ),
      new FunctionSyntax(__GetAccessUrl, "GetAccessUrl", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_STRING, "User Name", true),
        new ParameterSyntax( ParameterSyntax.PT_STRING, "Access Page", true),
        new ParameterSyntax( ParameterSyntax.PT_SYMBOL, "Target Symbol", true),
        new ParameterSyntax( ParameterSyntax.PT_INT, "Count", false),
      }
      ),
      new FunctionSyntax(__SendResetPasswordMail, "SendResetPasswordMail", new ParameterSyntax[] {
        new ParameterSyntax( ParameterSyntax.PT_STRING, "User Name", true),
        new ParameterSyntax( ParameterSyntax.PT_STRING, "EMail", true),
        new ParameterSyntax( ParameterSyntax.PT_STRING, "Password Change Page", true),
      }
      )
    }
    );
    
  }
  
  public SML(SecurityContext sctx, Config sdconfig, MoreMotionRequest request, Logger parent) {
    super(request, parent);
    this.sdconfig = sdconfig;
    this.sctx = sctx;
    this.request = request;
    this.secdom = sdconfig.getName();
  }
  
  public ADOMNode createMessage(String messageId, String[] arguments, Object details) {
    ADOMNode msg = request.createMessage("mor_security", messageId, arguments, details);
    msg.setNodeValue("secdom",secdom);
    return msg;
  }
  
  public ADOMNode createMessage(Exception exception) {
    ADOMNode msg = request.createMessage(exception );
    msg.setNodeValue("secdom",secdom);
    return msg;
  }
  
  public int executeFunction(int funcid, Object[] parms) throws CLException {
    
    int result = 0;
    
    try {
      switch (funcid) {
        case __LoginUser  : result = executeLoginUser( (String)parms[0], (String)parms[1], (Boolean)parms[2]); break;
        case __LogoutUser  : result = executeLogoutUser( (String)parms[0] ); break;
        case __CreateUser  : result = executeCreateUser( (String)parms[0], (String)parms[1], (String)parms[2], (String)parms[3], (Boolean)parms[4], (String)parms[5], (String)parms[6]); break;
        case __UpdateUser  : result = executeUpdateUser( (String)parms[0], (String)parms[1], (String)parms[2], (Boolean)parms[3], (String)parms[4]); break;
        case __RemoveUser  : result = executeRemoveUser( (String)parms[0] ); break;
        case __ChangePassword  : result = executeChangePassword( (String)parms[0], (String)parms[1], (String)parms[2]); break;
        case __ResetPassword  : result = executeResetPassword( (String)parms[0], (String)parms[1], (String)parms[2]); break;
        case __AddRoleToUser  : result = executeAddRoleToUser( (String)parms[0], (String)parms[1]); break;
        case __RemoveRoleFromUser  : result = executeRemoveRoleFromUser( (String)parms[0], (String)parms[1]); break;
        case __HasCheckPoint  : result = executeHasCheckPoint( (String)parms[0], (String)parms[1], (String)parms[2]); break;
        case __HasUserAuthority : result = executeHasUserAuthority( (String)parms[0], (String)parms[1], (Symbol)parms[2]); break;
        case __SetRole  : result = executeSetRole( (String)parms[0], (String)parms[1]); break;
        case __RemoveRole  : result = executeRemoveRole( (String)parms[0] ); break;
        case __AddCheckPointToRole  : result = executeAddCheckPointToRole( (String)parms[0], (String)parms[1]); break;
        case __RemoveCheckPointFromRole  : result = executeRemoveCheckPointFromRole( (String)parms[0], (String)parms[1]); break;
        case __SetCheckPoint  : result = executeSetCheckPoint( (String)parms[0], (String)parms[1]); break;
        case __RemoveCheckPoint  : result = executeRemoveCheckPoint( (String)parms[0]); break;
        case __SetSubdomain  : result = executeSetSubdomain( (String)parms[0] ); break;
        case __SetVar  : result = executeSetVar( (String)parms[0], (String)parms[1]); break;
        case __GetAccessUrl  : result = executeGetAccessUrl( (String)parms[0], (String)parms[1], (Symbol)parms[2], (String)parms[3]); break;
        case __SendResetPasswordMail  : result = executeSendResetPasswordMail( (String)parms[0], (String)parms[1], (String)parms[2]); break;
      }
    } catch (Exception e) {
      throw new CLException(e);
      
    }
    
    return result;
  }
  
  public LanguageSyntax getSyntax() {
    return lang;
  }
  
  private int executeLoginUser(String username, String password, Boolean multLogin) throws Exception {
    
    boolean pass_optional = password == null ? true : false;
    
    User user = null;
    
    try {
      user = sctx.getUser(username);
      if (user.isAccountDisabled()) {
        if (logger.LEVEL1) logger.printres("USER_ACCOUNT_DISABLED_LOG", username);
        createMessage("USER_ACCOUNT_DISABLED",new String[] {username}, null);
        return 8;
      }
      if (logger.LEVEL2) logger.printres("USER_ID_VALID_LOG",username);
      
      int suspendPeriod = sdconfig.getParameter("fla.suspend").resolve(request).intValue(5); // default, 5 min suspend
      int maxtry = sdconfig.getParameter("fla.maxtry").resolve(request).intValue(3); // default, 3 login attempt
      int flatries = 0;
      if (maxtry > 0) {
        flatries = sctx.flaTries(username, suspendPeriod * 1000L * 60);
      }
      
      if (maxtry > 0 && flatries >= maxtry) {
        createMessage("USER_SUSPENDED",new String[] {suspendPeriod + ""}, null);
        request.getVPool().put("user_suspended","true");
        return 8;
      }
      
      if (!pass_optional) {
        if (!user.authenticate(password)) {
          if (maxtry > 0) {
            flatries = sctx.registerFailedLoginAttempt(username,suspendPeriod * 1000L * 60);
            if (flatries >= maxtry) {
              createMessage("USER_SUSPENDED",new String[] {suspendPeriod + ""}, null);
              request.getVPool().put("user_suspended","true");
              return 8;
            }
          }
          
          if (maxtry > 0 && flatries+1 >= maxtry ) {
            createMessage("LAST_TRY_BEFORE_SUSPENSION",null, null);
            request.getVPool().put("last_try_before_suspension","true");
            if (logger.LEVEL1) logger.printres("PASSWORD_WRONG_BEFORE_SUSPENSION_LOG", username);
            return 8;
            
          } else {
            if (logger.LEVEL1) logger.printres("PASSWORD_WRONG_LOG", username);
            createMessage("WRONG_PASSWORD",new String[] {username}, null);
            return 8;
          }
        }
        if (logger.LEVEL2) logger.printres("USER_AUTHENTICATED_LOG", username);
      }
      
    } catch(MissingSecurityEntityException fe) {
      if (logger.LEVEL1) logger.printres("USER_ID_INVALID_LOG", username);
      createMessage("USER_NOT_FOUND",new String[] {username}, null);
      return 8;
    }
    
    sctx.login(user, multLogin == null ? true : multLogin.booleanValue());
    sctx.getRememberUser().login();
    
    if (logger.LEVEL2) logger.printres("USER_LOGGEDIN_LOG",username);
    
    String adomName = "mor_security_userinfo_" + secdom;
    ADOM uinfo = request.newSessionADOM(adomName);
    uinfo.setResBundleName("mor_security");
    Utils.populateUserInfo(uinfo, user, true);
    
    if (logger.LEVEL2) logger.printres("USERINFO_ADOM_CREATED_LOG",adomName);
    
    return 0;
  }
  
  private int executeLogoutUser(String userName) throws Exception {
    
    if (userName != null) {
      sctx.logout(userName);
      return 0;
    }
    
    User user = sctx.getCurrentUser();
    sctx.getRememberUser().setUser(user.getName(),sctx.getSubdomain());
    sctx.logout();
    sctx.getRememberUser().logout();
    
    request.removeSessionADOM("mor_security_userinfo_" + secdom);
    if (logger.LEVEL2) logger.printres("USERINFO_ADOM_REMOVED_LOG", "mor_security_userinfo_" + secdom);
    
    return 0;
  }
  
  private static void parseUserAttributes(String s, User user) {
    StringParser sp = new StringParser(s,';');
    while (!sp.eos()) {
      String def = sp.nextItem();
      String n = XString.before(def,'=', "");
      String v = XString.after(def,'=', "");
      user.setAttribute(n, v);
    }
  }
  
  private static final String validChars = "qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM_0123456789.@-";

  private boolean isValidUserName(String name) {
    for (int i = 0; i < name.length(); i++) {
      if ( validChars.indexOf(name.charAt(i)) == -1) return false;
    }
    return true;
  }
  
  private int executeCreateUser(String username, String password, String fullname,
      String email, Boolean disabled, String attrs, String poolvar) throws Exception {
    
    User user = null;
    try {
      user = sctx.getUser(username);
      createMessage("USER_ALREADY_EXISTS",new String[] {username},null);
      return 8;
    } catch (Exception ignored) {}
    
    if (!isValidUserName(username)) {
      createMessage("INVALID_USERNAME",new String[] {username},null);
      return 8;
    }
    
    if (email != null && email.length() > 0) {
      try {
        user = sctx.getUserByEmail(email);
        createMessage("EMAIL_ALREADY_EXISTS",new String[] {email},null);
        return 8;
      } catch (Exception ignored) {}
    }
    
    
    if (password.indexOf(' ') > -1) {
      createMessage("INVALID_PASSWORD",null,null);
      return 8;
    }
    
    user = sctx.newUser(username,  password, email, fullname, disabled.booleanValue() );
    
    // attributes
    if (attrs != null && attrs.length() > 0) {
      parseUserAttributes(attrs, user);
    }
    
    
    sctx.saveUser(user);
    
    // poolvar
    if (poolvar != null && poolvar.length() > 0) {
      request.getVPool().put(poolvar, user.getId());
    }
    
    return 0;
  }
  
  private int executeUpdateUser(String username, String fullname, String email, Boolean disabled, String attrs) throws Exception {
    
    User user = null;
    try {
      user = sctx.getUser(username);
    } catch (Exception e) {
      createMessage("USER_NOT_FOUND",new String[] {username},e);
      return 8;
    }
    
    // fullname
    if (fullname != null && fullname.length() > 0) {
      user.setFullname(fullname);
    }
    
    // email
    if (email != null && email.length() > 0) {
      try {
        User userx = sctx.getUserByEmail(email);
        if ( email.trim().length() > 0 && !user.getName().equals( userx.getName() ) ) {
          createMessage("EMAIL_ALREADY_EXISTS",new String[] {email},null);
          return 8;
        }
      } catch (Exception ignored) {}
      
      user.setEmail(email);
    }
    
    // disabled
    if (disabled != null) {
      user.setAccountDisabled(disabled.booleanValue());
    }
    
    // attributes
    if (attrs != null && attrs.length() > 0) {
      parseUserAttributes(attrs, user);
    }
    
    sctx.saveUser(user);
    
    //    User currentUser = sctx.getCurrentUser();
    //    if (currentUser != null && currentUser.getName().equals(user.getName())) {
    //      user.setLoggedIn(true);
    //      sctx.setCurrentUser(user);
    //    }
    
    User currentUser = sctx.getCurrentUser();
    if (currentUser != null && user.getName().equals(currentUser.getName())) {
      currentUser = sctx.getCurrentUser(true);
      String adomName = "mor_security_userinfo_" + secdom;
      ADOM uinfo = request.newSessionADOM(adomName);
      uinfo.setResBundleName("mor_security");
      user = sctx.getUser(user.getName()); // Reload User Info from DB
      Utils.populateUserInfo(uinfo, user, true);
      if (logger.LEVEL2) logger.printres("USERINFO_ADOM_REFRESHED_LOG", adomName);
    }
    
    return 0;
  }
  
  private int executeRemoveUser(String username) throws Exception {
    
    User user = null;
    try {
      user = sctx.getUser(username);
    } catch (Exception e) {
      createMessage("USER_NOT_FOUND",new String[] {username},e);
      return 8;
    }
    
    sctx.deleteUser(username);
    
//    EventQ q = sdconfig.getEventQ("OnRemoveUserEvent");
//    if (q.hasListener()) {
//      OnRemoveUserEvent event = new OnRemoveUserEvent(request,user);
//      q.dispatch(event);
//    }
    
    return 0;
  }
  
  private int executeChangePassword(String password, String newpassword, String newpasswordagain) throws Exception {
    
    User user = sctx.getCurrentUser();
    if (user == null) {
      createMessage("NO_CURRENT_USER", null, null);
      return 8;
    }
    
    if (!user.authenticate(password)) {
      createMessage("OLD_PASSWORD_WRONG", new String[] {user.getName()}, null);
      return 8;
    }
    
    if (!newpassword.equals(newpasswordagain)) {
      createMessage("NEW_PASSWORDS_NOT_MATCHED",new String[] {"ChangePassword()"}, null);
      return 8;
    }
    
    user.setPassword(newpassword);
    sctx.saveUser(user);
    
    return 0;
  }
  
  private int executeResetPassword(String username, String newpassword, String newpasswordagain) throws Exception {
    
    User currentUser = sctx.getCurrentUser();
    
    if (username.equals(sctx.getRootUserName())) {
      if (currentUser == null || !currentUser.getName().equals(username)) {
        createMessage("CANNOT_CHANGE_ROOT_PASSWORD", null, null);
        return 8;
      }
    }
    
    if (!newpassword.equals(newpasswordagain)) {
      createMessage("NEW_PASSWORDS_NOT_MATCHED",new String[] {"ResetPassword()"}, null);
      return 8;
    }
    
    User user = null;
    
    if (username.trim().length() == 0) {
      username = (String)request.getSession().getAttribute("temporary.loggedin");
      request.getSession().removeAttribute("temporary.loggedin");
      if (username == null || username.trim().length() == 0) return 0;
    }
    
    try {
      user = sctx.getUser(username);
    } catch (Exception e) {
      createMessage("USER_NOT_FOUND",new String[] {username},e);
      return 8;
    }
    
    user.setPassword(newpassword);
    sctx.saveUser(user);
    
    return 0;
  }
  
  
  private int executeAddRoleToUser(String username, String rolename) throws Exception {
    
    User user = null;
    try {
      user = sctx.getUser(username);
    } catch (Exception e) {
      createMessage("USER_NOT_FOUND",new String[] {username},e);
      return 8;
    }
    
    Role role = null;
    try {
      role = sctx.getRole(rolename);
    } catch (Exception e) {
      createMessage("ROLE_NOT_FOUND",new String[] {rolename},e);
      return 8;
    }
    
    user.addRole(role);
    sctx.saveUser(user);
    
    return 0;
  }
  
  private int executeRemoveRoleFromUser(String username, String rolename) throws Exception {
    
    User user = null;
    try {
      user = sctx.getUser(username);
    } catch (Exception e) {
      createMessage("USER_NOT_FOUND",new String[] {username},e);
      return 8;
    }
    
    Role role = null;
    try {
      role = sctx.getRole(rolename);
      user.removeRole(role);
      sctx.saveUser(user);
    } catch (Exception ignored) {}
    
    return 0;
  }
  
  private int executeHasUserAuthority(String username, String cpname, Symbol symbol) throws Exception {
    
    User user = null;
    try {
      user = sctx.getUser(username);
    } catch (Exception e) {
      createMessage("USER_NOT_FOUND",new String[] {username},e);
      return 8;
    }
    
    boolean auth = sctx.hasAuthority(user, cpname);
    
    MMSymbolResolver mmsr = request.getSymbolResolver();
    mmsr.setValue(symbol.toString(), "" + auth);
    if (logger.LEVEL3) {
      logger.printres("SYMBOL_IS_SET_TO", symbol.toString(), auth+"");
    }
    
    return 0;
  }
  
  private int executeGetAccessUrl(String username, String accessPage, Symbol symbol, String temporary) throws Exception {
    
    User user = null;
    try {
      user = sctx.getUser(username);
    } catch (Exception e) {
      createMessage("USER_NOT_FOUND",new String[] {username},e);
      return 8;
    }
    
    String subdomain = sctx.getSubdomain();
    boolean temp = temporary != null && temporary.equalsIgnoreCase("true");
    String enckey = sdconfig.getParameter("fpmail.enckey").stringValue("");
    String url = Utils.getAccessUrl(request,secdom,user,subdomain,accessPage,enckey,temp);
    
    MMSymbolResolver mmsr = request.getSymbolResolver();
    mmsr.setValue(symbol.toString(), url );
    if (logger.LEVEL3) {
      logger.printres("SYMBOL_IS_SET_TO", symbol.toString(), url);
    }
    
    return 0;
  }
  
  
  private int executeSendResetPasswordMail(String username, String email, String resetPage) throws Exception {
    
    User user = null;
    
    if (email.length() > 0) {
      try {
        user = sctx.getUserByEmail(email);
      } catch (Exception e) {
        createMessage("NO_SUCH_USER",new String[] {email},e);
        return 8;
      }
      
    } else {
    
      try {
        user = sctx.getUser(username);
        email = user.getEmail();
        if (email == null || email.length() == 0) {
          createMessage("NO_EMAIL_DEFINED",new String[] {username},null);
          return 8;
        }
      } catch (Exception e) {
        createMessage("NO_SUCH_USER",new String[] {username},e);
        return 8;
      }
    }
    
    String subdomain = sctx.getSubdomain();
    boolean temp = true;
    String enckey = sdconfig.getParameter("fpmail.enckey").stringValue("");
    String url = Utils.getAccessUrl(request,secdom,user,subdomain,resetPage,enckey,temp);
    
    ForgotPassword.sendMessage(request, sdconfig, email, url);
    
    return 0;
  }
  
  
  private int executeHasCheckPoint(String username, String cpname, String symbol) throws Exception {
    
    User user = null;
    try {
      user = sctx.getUser(username);
    } catch (Exception e) {
      createMessage("USER_NOT_FOUND",new String[] {username},e);
      return 8;
    }
    
    boolean auth = sctx.hasAuthority(user, cpname);
    
    request.getSymbolResolver().setValue(symbol,""+auth);
    
    return 0;
  }
  
  private int executeSetRole(String rolename, String roledesc) throws Exception {
    
    Role role = null;
    try {
      role = sctx.getRole(rolename);
      role.setDescription(roledesc);
    } catch (Exception e) {
      role = new Role(rolename, roledesc);
    }
    sctx.saveRole(role);
    
    return 0;
  }
  
  private int executeRemoveRole(String rolename) throws Exception {
    
    sctx.deleteRole(rolename);
    
    return 0;
  }
  
  private int executeAddCheckPointToRole(String rolename, String cpname) throws Exception {
    
    Role role = null;
    try {
      role = sctx.getRole(rolename);
    } catch (Exception e) {
      createMessage("ROLE_NOT_FOUND",new String[] {rolename},e);
      return 8;
    }
    
    CheckPoint cp = null;
    try {
      cp = sctx.getCheckPoint(cpname);
    } catch (Exception e) {
      createMessage("CHECKPOINT_MISSING",new String[] {cpname},e);
      return 8;
    }
    
    role.addCheckPoint(cp);
    sctx.saveRole(role);
    
    return 0;
  }
  
  private int executeRemoveCheckPointFromRole(String rolename, String cpname) throws Exception {
    
    Role role = null;
    try {
      role = sctx.getRole(rolename);
    } catch (Exception e) {
      createMessage("ROLE_NOT_FOUND",new String[] {rolename},e);
      return 8;
    }
    
    CheckPoint cp = null;
    try {
      cp = sctx.getCheckPoint(cpname);
    } catch (Exception e) {
      createMessage("CHECKPOINT_MISSING",new String[] {cpname},e);
      return 8;
    }
    
    role.removeCheckPoint(cp);
    sctx.saveRole(role);
    
    return 0;
  }
  
  private int executeSetCheckPoint(String cpname, String cpdesc) throws Exception {
    
    CheckPoint cp = null;
    try {
      cp = sctx.getCheckPoint(cpname);
      cp.setDescription(cpdesc);
    } catch (Exception e) {
      cp = new CheckPoint(cpname, cpdesc);
    }
    sctx.saveCheckPoint(cp);
    
    return 0;
  }
  
  private int executeRemoveCheckPoint(String cpname) throws Exception {
    
    sctx.deleteCheckPoint(cpname);
    
    return 0;
  }
  
  private int executeSetVar(String name, String value) throws Exception {
    
    sctx.setVar(name, value);
    
    return 0;
  }
  
  
  private int executeSetSubdomain(String subdomain) throws Exception {
    
    sctx.setSubdomain(subdomain);
    
    return 0;
  }
  
  public static void main(String[] args) throws Exception {
  }
  
}