/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.mmcomp.security;

import java.io.IOException;
import org.moremotion.action.ActionServiceContext;

import org.moremotion.adom.ADOM;
import org.moremotion.security.MissingSecurityEntityException;
import org.moremotion.security.SecurityContext;
import org.moremotion.security.User;
import org.moremotion.action.ActionService;
import org.moremotion.servlet.ServiceException;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.config.Config;
import org.moremotion.servlet.SystemContext;
import org.moremotion.util.StringParser;
import org.moremotion.util.XString;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: SignIn.java 51 2008-03-10 11:46:54Z erkan $
 */
public class SignIn implements ActionService {
  
  public void doService(ActionServiceContext asc) throws ServiceException, IOException {
    
    try {
      
      MoreMotionRequest request = asc.getRequest();
      String secdom = request.getParameter("secdom","main");
      
      Config sdconfig = Utils.getSecurityDomainConfig(secdom);
      SecurityContext sctx = Utils.getSecurityContext(request, sdconfig);
      
      String enckey = sdconfig.getParameter("fpmail.enckey").stringValue("");
      String key = request.getParameter("key","");
      key = SystemContext.encryptor().decrypt(key,"accurl" + enckey);
      if (key.equals("DECRYPT_ERROR!")) {
        request.generatePage("index");
        return;
      }
      
      String userName = "";
      String subDomain = "";
      String password = "";
      String nextPage = "";
      String temporary = null;
      
      StringParser parser = new StringParser(key,'&');
      while (parser.hasMore()) {
        String def = parser.nextItem();
        String prm = XString.before(def,'=',null);
        String val = XString.after(def,'=',null);
        if (prm.equals("username")) userName = val;
        else if (prm.equals("subdomain")) subDomain = val;
        else if (prm.equals("password")) password = val;
        else if (prm.equals("nextpage")) nextPage = val;
        else if (prm.equals("temporary")) temporary = val;
      }
      
      if (userName.equals("")) {
        request.generatePage("index");
        return;
      }
      
      User user = null;
      try {
        sctx.setSubdomain(subDomain);
        
        user = sctx.getUser(userName);
        if (user.isAccountDisabled()) {
          request.generatePage("index");
          return;
        }
        
        if (!user.getPassword().equals(password)) {
          request.generatePage("index");
          return;
        }
        
      } catch(MissingSecurityEntityException fe) {
        request.generatePage("index");
        return;
      }
      
      sctx.login(user);
      
      String adomName = "mor_security_userinfo_" + secdom;
      ADOM adom = request.newSessionADOM( adomName );
      adom.setResBundleName("mor_security");
      
      Utils.populateUserInfo(adom,user,true);
      
      request.setAllowCaching(-1);
      
      request.generatePage(nextPage);
      
      if (temporary != null && temporary.equalsIgnoreCase("true")) {
        sctx.logout();
        request.getSession().setAttribute("temporary.loggedin",user.getName());
      }
      
    } catch (Exception e) {
      throw new ServiceException(e);
    }
    
    
  }
  
  
}