/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */


package org.moremotion.mmcomp.security;

import java.util.Enumeration;
import java.util.Properties;
import org.moremotion.datasrc.DataService;
import org.moremotion.datasrc.DataServiceContext;
import org.moremotion.datasrc.Paging;
import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMNode;
import org.moremotion.config.Config;
import org.moremotion.security.SecurityContext;
import org.moremotion.security.User;
import org.moremotion.servlet.ServiceException;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.MutableInt;


/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: UserList.java 51 2008-03-10 11:46:54Z erkan $
 */
public class UserList implements DataService {
  
  public void doService(ADOM adom, DataServiceContext dsc) throws ServiceException {
    
    try {
      
      MoreMotionRequest request = dsc.getRequest();
      String secdom = Utils.getSecurityDomainName(request, dsc.getDataSourceConfig());
      Config sdconfig = Utils.getSecurityDomainConfig(secdom);
      SecurityContext sctx = Utils.getSecurityContext(request, sdconfig);
      
      String searchCriteria = "";
      ADOM criteria = request.getSessionADOM("mor_security_searchcriteria_" + secdom );
      if (criteria != null) {
        searchCriteria = criteria.getNodeValue("_criteria");
      }
      
      MutableInt noi = new MutableInt(999999);
      int amount = 999999;
      User[] users = null;
      
      Paging paging = dsc.getPaging();
      
      if (paging.getItemsPerPage() > 0) {
        
        amount = paging.getItemsPerPage();
        if (searchCriteria != null && searchCriteria.length() != 0) {
          users = sctx.searchUsers(searchCriteria, paging.getSkipAmount(), amount, noi);
        } else {
          users = sctx.getUsers( paging.getSkipAmount(), amount, noi );
        }
        
        if (users.length == 0) {
          paging.setNumberOfItems(noi.value);
          if (searchCriteria != null && searchCriteria.length() != 0) {
            users = sctx.searchUsers(searchCriteria, paging.getSkipAmount(), amount, noi);
          } else {
            users = sctx.getUsers( paging.getSkipAmount(), amount, noi );
          }
        }
        
        paging.setNumberOfItems(noi.value);
        // paging.setMorePages(true);
      } else {
        if (searchCriteria != null && searchCriteria.length() != 0) {
          users = sctx.searchUsers(searchCriteria, 0, amount, noi);
        } else {
          users = sctx.getUsers();
        }
      }
      
      for (int i = 0; i < users.length; i++) {
        User user = users[i];
        ADOMNode usernode = new ADOMNode("item");
        usernode.setNodeValue("id",user.getId());
        usernode.setNodeValue("username",user.getName());
        usernode.setNodeValue("fullname",user.getFullname());
        usernode.setNodeValue("email",user.getEmail());
        usernode.setNodeValue("disabled","" + user.isAccountDisabled());
        
        Properties attrs = user.getAttributes();
        if (attrs != null) {
          Enumeration en = attrs.propertyNames();
          while (en.hasMoreElements()) {
            try {
              String attrName = (String)en.nextElement();
              String attrValue = attrs.getProperty(attrName);
              usernode.setNodeValue(attrName, attrValue);
            } catch (Exception e){}
          }
        }
        adom.addNode(usernode);
      }
    } catch (Exception e) {
      throw new ServiceException(e);
    }
  }
  
  
}
