/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.action;

import java.util.HashMap;
import org.moremotion.config.Config;
import org.moremotion.config.ConfigNode;
import org.moremotion.config.ConfigNodeList;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.config.ConfigUnit;
import org.moremotion.config.ConfigurationException;
import org.moremotion.security.AccessControlConfigParameter;
import org.moremotion.security.AccessControlDefinition;
import org.moremotion.servlet.MoreMotionRequest;

/**
 * MoreMotion Action Service Configuration.
 *
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ActionServiceConfig.java 51 2008-03-10 11:46:54Z erkan $
 */
public class ActionServiceConfig extends Config {
  
  private String className = null;
  
  private ConfigParameter unitname = null;
  
  private AccessControlConfigParameter accessControl = null;
  
  /**
   * The debugging level from 1 to 3
   */
  protected ConfigParameter debug = null;
  
  public ActionServiceConfig(ConfigUnit unit) throws ConfigurationException {
    super(unit,false);
    try {
      parse(unit.getConfigNodeList());
    } catch (Exception e) {
      throw new ConfigurationException(e);
    }
  }
  
  private void parse(ConfigNodeList nodelist) throws ConfigurationException {
    
    for (int i = 0; i < nodelist.size(); i++) {
      
      ConfigNode node = nodelist.get(i);
      
      String name = node.getName();
      
      if (name.equals("accessControl")) {
        accessControl = new AccessControlConfigParameter(node.getElement());
      }
      
      else if (name.equals("unitname")) {
        unitname = new ConfigParameter("unitname", node.getValue());
      }
      
      else if (name.equals("debug")) {
        debug = new ConfigParameter("debug", node.getValue());
      }
      
      else if (name.equals("class")) {
        className = node.getValue();
      }
      
      else {
        if (params == null) params = new HashMap();
        params.put(name, new ConfigParameter(name, node.getValue())) ;
      }
      
      
    }
    
    if (debug == null) {
      debug = new ConfigParameter("debug", "0");
    }
    
    if (className == null) {
      throw new ConfigurationException("ACTION_SERVICE_CLASS_NAME_MISSING",new String[]{getName()},null);
    }    
    
  }
  
  
  public String getUnitName(MoreMotionRequest request) throws ConfigParameterException {
    return unitname == null ? null : unitname.resolve(request).stringValue(null);
  }
  
  public AccessControlDefinition getAccessControlDefinition(MoreMotionRequest request) throws ConfigParameterException {
    return accessControl == null ? null : accessControl.getAccessControlDefinition(request);
  }
  
  
  public String getClassName() {
    return this.className;
  }
  
  
}