/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.action;

import java.util.HashMap;
import org.moremotion.config.ConfigCache;
import org.moremotion.config.ConfigNodeList;
import org.moremotion.config.ConfigUnit;
import org.moremotion.config.ConfigurationException;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.servlet.ServiceException;
import org.moremotion.servlet.ServiceException;
import org.moremotion.security.AccessControlDefinition;
import org.moremotion.security.SecurityManager;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ActionServiceManager.java 193 2008-08-14 16:09:44Z erkan $
 */
public class ActionServiceManager {
  
  private static HashMap serviceCach = new HashMap();
  
  private ActionServiceManager() {}
  
  public static ActionServiceConfig getActionServiceConfig(MoreMotionRequest request, String name) throws ConfigurationException {
    
    String originPage = request.getOriginPage();
    
    ConfigUnit unit = ConfigCache.getUnit("action", originPage + '.' + name);
    if (unit == null) {
      unit = ConfigCache.getUnit("action", name);
      if (unit == null) return null;
    }
    
    if (unit.isNew()) {
      synchronized (unit.getSyncObject()) {
        parse(unit);
      }
    }
    
    return (ActionServiceConfig)unit.getDataObject();
  }
  
  private static void parse(ConfigUnit unit) throws ConfigurationException {
    
    ConfigNodeList nodelist = unit.getConfigNodeList();
    
    // ActionServiceConfig asconfig = new ActionServiceConfig(unit.getName(), nodelist);
    ActionServiceConfig asconfig = new ActionServiceConfig(unit);
    unit.setDataObject(asconfig);
    
  }
  
  
  public static void runActionService(String serviceName, MoreMotionRequest request) throws Exception  {
    
    ActionServiceContext asc = new ActionServiceContext();
    
    asc.setRequest(request);
    
    ActionServiceConfig asconfig = getActionServiceConfig(request, serviceName);
    
    if (asconfig != null) {
      serviceName = asconfig.getClassName();
      asc.setActionServiceConfig(asconfig);
      
      /*
       * Check Access Authority
       */
      AccessControlDefinition acd = asconfig.getAccessControlDefinition(request);
      if (acd != null) {
        try {
          if (SecurityManager.checkAccessAuthority(request, acd, null) == false) {
          /*
           * SecurityManager already redirected to another page. So just return.
           */
            return;
          }
        } catch (Exception ex) {
          throw new ServiceException(ex);
        }
      }
      
      asc.setLogger(asconfig.getName(), asconfig.debug.resolve(request).intValue(0), asconfig.getUnitName(request));
    }
    
    ActionService service = getServiceClass(serviceName);
    service.doService(asc);
    
  }
  
  private static ActionService getServiceClass(String className) throws ServiceException {
    
    Object object = serviceCach.get(className);
    
    if (object == null) {
      synchronized (serviceCach) {
        object = serviceCach.get(className);
        if (object == null) {
          try {
            object = (Object) Class.forName(className).newInstance();
          } catch (Exception e) {
            throw new ServiceException("ACTION_SERVICE_CLASS_MISSING",new String[]{className},e);
          }
          serviceCach.put(className, object);
        }
      }
    }
    
    return (ActionService)object;
  }
  
  
}
