/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.adom;

import org.moremotion.exception.MoreMotionException;

/** 
 * Indicates that something went wrong while generating an ADOM object.
 * <P>While generating an ADOM, you can redirect page generator to another
 * page by setting redirect page property.</P>
 *
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ADOMException.java 51 2008-03-10 11:46:54Z erkan $
 *
 */
public class ADOMException extends MoreMotionException {  

  private String redirectPage = null;

  /** 
   * Creates a new ADOMException using a root cause
   * @param rootCause The Exception that led to this ADOMException
   */  
  public ADOMException(Exception rootCause) { 
    super(rootCause); 
  }

  /** 
   * Creates a new ADOMException using a message text
   * @param message The message test
   */  
  public ADOMException(String message) { 
    super(message); 
  } 
  
  /** 
   * Creates a new ADOMException.
   * @param message
   */   
  public ADOMException(String message, String[] arguments, Exception rootCause) {
    super(message, arguments, rootCause);
  }

  /** 
   * Creates a new ADOMException and sets the page to redirect.
   * @param message Exception message.
   * @param redirectPage Page to redirect.
   */
  public ADOMException(String message, String redirectPage) {
    super(message,null,null);
    this.redirectPage = redirectPage;
  }
  
  /** 
   * Returns redirected page.
   */  
  public String getRedirectedPage() {
    return this.redirectPage;
  }
  
  private void setRedirectPage(String redirectPage) {
    this.redirectPage = redirectPage;
  }
  
  /** 
   * Returns if redirection is required.
   */  
  public boolean isRedirectionRequired() {
    return (this.redirectPage != null);
  }
  
  /** 
   * Utility method for throwing an ADOM Exception with a redirection
   * to a specified page.
   */
  public static void redirect(String page) throws ADOMException {
    ADOMException ae = new ADOMException("Page redirection required for " + page);
    ae.setRedirectPage(page);
    throw ae;
  }
}