/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */


package org.moremotion.cache;

import java.io.BufferedOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import org.moremotion.util.Logger;
import org.moremotion.servlet.SystemConfig;
import org.moremotion.servlet.SystemContext;
import org.moremotion.util.XString;


/**
 * Responsible for unzipping the files with extension *.mmzip found under application root folder and in sub folders.
 * The unpacked files from the zip file is logged in a file on th same directory as the zipfile with 
 * extension *.mmzip.unzip.history
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ZipDeployer.java 51 2008-03-10 11:46:54Z erkan $
 */
public class ZipDeployer implements Runnable {
  
  private Thread t;
  private boolean stop = false;
  private long unique = 0;
  private File root = null;
  
  public ZipDeployer() {
    this.root = new File(SystemContext.servletContext().getRealPath("/"));
    t = new Thread(this);
    t.setDaemon(true);
    t.start();
  }
  
  
  public void run() {
    while(!stop) {
      try {
        synchronized (this) {
          
          List files = getFileListing(this.root);
          if (files != null) {
            for (int i = 0; i < files.size(); i++) {
              File f = (File) files.get(i);
              unzip(f);
            }
          }
          
        }
        t.sleep(1000L * 2);  // sleep for 2 seconds
      } catch(Exception ex) {
        ex.printStackTrace();
      }
    }
  }
  
  public void stop() {
    this.stop = true;
  }
  
  public void finalize() {
    stop();
  }
  
  private void unzip(File zipfile) throws Exception {
    
    SystemContext.logger().printres("UNZIPPING_MMZIP",zipfile.getName());
    
    File histFile = new File(zipfile.getAbsolutePath() + ".unzip.history");
    
    FileOutputStream fos = new FileOutputStream(histFile,true);
    DataOutputStream hist = new DataOutputStream(fos);
    
    SimpleDateFormat format = new SimpleDateFormat("yyyy.MM.dd HH:mm");
    Date d = new Date();
    
    hist.writeBytes("Unzipping " + zipfile.getName() + " at " + format.format(d) + "\n");
    
    ZipFile zip = new ZipFile(zipfile);
    
    File outdir = zipfile.getParentFile();
    
    Enumeration e = zip.entries();
    
    outdir.mkdirs();
    
    ZipEntry zipentry;
    
    while( e.hasMoreElements() ) {
      
      zipentry = (ZipEntry)e.nextElement();
      if( zipentry.isDirectory() ) {
        
        File dir = new File( outdir, zipentry.getName());
        if( !dir.exists() )
          dir.mkdirs(); //create subdirectories
        
      } else {
        
        hist.writeBytes("Unpacking " + zipentry.getName() + "\n");

        InputStream in = zip.getInputStream(zipentry);
        File outfile = new File(outdir, zipentry.getName());
        outfile.getParentFile().mkdirs();
        OutputStream out = new BufferedOutputStream( new  FileOutputStream( outfile ) );
        int x;
        while( (x = in.read()) != -1 ) {
          out.write(x);
        }
        out.close();
        in.close();
      }
    }
    
    zip.close();
    hist.flush();
    hist.close();
    
  }
  
  private List getFileListing( File startDir ) {
    
    ArrayList list = new ArrayList();
    collectZipFiles(startDir,list);
    return list;
    
  }
  
  private void collectZipFiles(File dir, ArrayList list) {
    
    if (!dir.exists()) return;
    
    File[] files = dir.listFiles(new ZipFileFilter());
    if (files == null || files.length == 0) {
      return;
    }
    
    File f = null;
    for (int i = 0; i < files.length; i++) {
      f = files[i];
      if (f.isFile()) {
        list.add(f);
      }
      if (f.isDirectory()) {
        collectZipFiles(f,list);
      }
      
    }
    return;
  }
  
  
  
  
    
  class ZipFileFilter implements java.io.FileFilter {
    
    public boolean accept(File file) {
      if (file.isDirectory()) {
        return true;
      }
      String name = file.getName();
      if (name.endsWith(".mmzip")) {
        File compareFile = new File(file.getAbsoluteFile() + ".unzip.history");
        if (file.lastModified() > compareFile.lastModified()) {
          return true;
        }
      }
      return false;
    }
    
  }
  
}
