/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * ConfigUnit.java
 *
 */

package org.moremotion.config;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ConfigUnit.java 115 2008-04-08 11:56:01Z erkan $
 */
public class ConfigUnit {
  
  protected String name = null;
  protected String type = null;
  protected String pageName = null;
  protected String baseUnitName = null;
  protected ConfigUnit baseUnit = null;
  protected String refId = null;
  protected int pathLength = -1;
  protected Element elm = null;
  protected Object data = null;
  
  
  public ConfigUnit(Element elm, String refId) {
    this.elm = elm;
    this.refId = refId;
    this.type = elm.getTagName();
    this.name = elm.getAttribute("name");
    String s = elm.getAttribute("basedon");
    if (s.length() > 0) this.baseUnitName = s;
  }
  
  public Element getElement() {
    return elm;
  }
  
  protected void clear() {
    this.baseUnit = null;
    this.data = null;
  }
  
  public void setDataObject(Object data) {
    this.data = data;
  }
  
  public Object getDataObject() {
    return this.data;
  }
  
  public String getPageName() {
    return this.pageName;
  }
  
  public boolean isNew() {
    
    if (data == null) {
      return true;
    }
    
    if (baseUnitName == null) {
      return false;
    }
    
    if (baseUnit == null) {
      return true;
    }
    
    return baseUnit.isNew();
  }
  
  public Object getSyncObject() {
    
    if (baseUnit == null) {
      return this.elm;
    }
    
    return baseUnit.getSyncObject();
  }
  
  
  public String getName() {
    return this.name;
  }
  
  public ConfigNodeList getConfigNodeList() throws ConfigurationException {
    ConfigNodeList nodelist = new ConfigNodeList();
    collectNodes(nodelist);
    return nodelist;
  }
  
  private void collectNodes(ConfigNodeList nodelist) throws ConfigurationException {
    
    if (baseUnitName != null) {
      
      if (baseUnit == null) {
        baseUnit = ConfigCache.getUnit(type, baseUnitName);
        if (baseUnit == null) {
          throw new ConfigurationException("BASE_CONFIG_UNIT_MISSING", new String[]{baseUnitName,name}, null);
        }
      }
      
      baseUnit.collectNodes(nodelist);
    }
    
    NodeList nlist = elm.getChildNodes();
    for (int i = 0; i < nlist.getLength(); i++) {
      Node node = nlist.item(i);
      if (node.getNodeType() == Node.ELEMENT_NODE) {
        nodelist.addLast(node);
      }
    }
    
  }
  
  
  public String getBaseUnitName() {
    return this.baseUnitName;
  }
  
  
  public String getRefId() {
    return this.refId;
  }
  
//  public Element getElement(String name) {
//    NodeList nlist = elm.getElementsByTagName(name);
//    if (nlist.getLength() == 0 && baseUnit != null) {
//      return baseUnit.getElement(name);
//    }
//    return (Element)nlist.item(0);
//  }
  
  
}
