/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * MScriptFunctionManager.java
 *
 */

package org.moremotion.evaluator;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import org.moremotion.adom.ADOM;
import org.moremotion.config.ConfigCache;
import org.moremotion.config.ConfigNodeList;
import org.moremotion.config.ConfigUnit;
import org.moremotion.config.ConfigurationException;
import org.moremotion.page.PageDataRequest;
import org.moremotion.datasrc.DataService;
import org.moremotion.servlet.MoreMotionRequest;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: MScriptFunctionManager.java 186 2008-07-31 18:23:46Z erkan $
 */

public class MScriptFunctionManager {
  
  private static HashMap functionCache = new HashMap();
  
  
  private MScriptFunctionManager() {}
  
  public static MScriptFunctionConfig getMScriptFunctionConfig(String name) throws ConfigurationException {
    
    ConfigUnit unit = ConfigCache.getUnit("mscriptFunction", name);
    if (unit == null) {
      return null;
      // throw new ConfigurationException("CONFIG_UNIT_MISSING",new String[]{name},null);
    }
    
    if (unit.isNew()) {
      synchronized (unit.getSyncObject()) {
        parse(unit);
      }
    }
    
    return (MScriptFunctionConfig)unit.getDataObject();
  }
  
  private static void parse(ConfigUnit unit) throws ConfigurationException {
    
    ConfigNodeList nodelist = unit.getConfigNodeList();
    
    // MScriptFunctionConfig msfconfig = new MScriptFunctionConfig(unit.getName(), nodelist);
    MScriptFunctionConfig msfconfig = new MScriptFunctionConfig(unit);
    unit.setDataObject(msfconfig);
    
  }
  
  protected static MScriptFunction loadFunction(String className) throws ConfigurationException {
    
    MScriptFunction func = (MScriptFunction) functionCache.get(className);
    
    if (func == null) {
      
      synchronized (functionCache) {
        
        try {
          func = (MScriptFunction) Class.forName(className).newInstance();
          functionCache.put(className, func);
          
        } catch (Exception ex) {
          throw new ConfigurationException("UNABLE_TO_LOAD_MSCRIPT_FUNCTION_CLASS",new String[]{className},ex);
        }
        
      }
    }
    
    return func;
  }
  
  
  public static void clearCache() {
    functionCache = new HashMap();  
  }
  
  
}
