/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * CurrentPage.java
 *
 * Created on 01 November 2005 Tuesday, 21:20
 */

package org.moremotion.page;

import org.moremotion.adom.ADOM;
import org.moremotion.config.ConditionalList;
import org.moremotion.config.ConditionalListIterator;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.servlet.SystemConfig;
import org.moremotion.util.FileUtils;
import org.moremotion.util.XString;


/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: CurrentPage.java 146 2008-06-04 21:23:59Z erkan $
 */
public class CurrentPage {
  
  private PageInfo pageInfo = null;
  private String uri = null;
  private MoreMotionRequest request;
  private String nextPages = "";
  private String page;
  private char pagePathSeparator = ' ';
  
  public CurrentPage(MoreMotionRequest request, String pageName) throws PageGenerationException {
    
    String s = request.getPlainParameter("_pgsep"); // pgsep: "Page-Separator" Request Parameter
    if (s != null) pagePathSeparator = s.charAt(0);
    
    this.request = request;
    String pg = pageName.replace('\\','/');
    if (pagePathSeparator != ' ') pg = pg.replace(pagePathSeparator,'/');
    
    this.page = actualPageName(pg,this.nextPages);
    ADOM adom = request.getRequestADOM("nextpage");
    if (adom != null) {
      try {
        insertNextPage(adom.getNodeValue("page"));
      } catch (Exception e) {}
    }
    arrangeWithLocation();
  }
  
  private String encoding = null;
  
  /**
   * Returns the encoding of the current page
   */
  public String getEncoding() throws ConfigParameterException {
    if (encoding == null) {
      encoding = pageInfo.getPageConfig().getEncoding(request);
      if (encoding == null) {
        encoding = request.getPlainParameter("_enc");
      }
      if (encoding == null || encoding.length() == 0) encoding = "iso-8859-1";
    }
    return encoding;
  }
  
  private static String actualPageName(String pg, String nextPages) throws PageGenerationException {
    
    String page = new String(pg);
    
    if (nextPages != null) nextPages = XString.after(page,',',"");
    page = XString.before(page,',');
    
    String pageofvalue = null;
    
    if (page.startsWith("s:page-of-")) {
      
      pageofvalue = SystemConfig.getParameter(page.substring(2),"");
      if (pageofvalue == null || pageofvalue.length() == 0) {
        throw new PageGenerationException("SYSTEM_PARAMETER_NOT_FOUND",new String[]{page},null);
      }
      page = pageofvalue;
    }
    
    return page;
  }
  
  private void arrangeWithLocation() {
    
    if (!page.startsWith("/")) {
      String location = request.getPageLocation();
      
      if (location != null && location.length() > 0) {
        location = location.replace('\\','/');
        if (pagePathSeparator != ' ') location = location.replace(pagePathSeparator,'/');
        if (location.charAt(0) != '/') location = "/" + location;
        if (!location.endsWith("/")) location = location + "/";
        page = location + page;
        
      } else {
        page = "/" + page;
      }
    }
  }
  
  
  public String getPageName() {
    return this.page;
  }
  
  public String getPageName(boolean withVariant) throws PageGenerationException {
    String result = getPageName();
    PageInfo pi = getPageInfo();
    if (pi != null && pi.isVariant()) {
      String vari = getVariant();
      if (vari != null && vari.trim().length() != 0 && !vari.equals("*")) {
        result = result += "/_." + vari;
      }
    }
    return result;
  }
  
  
  public String getNextPages() {
    if (nextPages.length() > 0)
      return page + "," + nextPages;
    else return page;
  }
  
  /**
   * Returns the Current CommandDefinition object associated with the specified command
   */
  public CommandDefinition getCurrentCommandDefinition() throws PageGenerationException {
    
    String cmd = request.getPlainParameter("_cmd");
    if (cmd == null) {
      cmd = request.getPlainParameter("_cdef");
    }
    
    return getCommandDefinition(cmd);
  }
  
  /**
   * Returns the CommandDefinition object associated with the specified command
   * @param command The name of the command
   */
  public CommandDefinition getCommandDefinition(String command) throws PageGenerationException {
    
    if (command == null) return null;
    
    ConditionalList clist = pageInfo.getPageConfig().getCommandDefinitions();
    if (clist == null) return null;
    
    try {
      ConditionalListIterator cli = clist.iterator(request);
      while (cli.hasMore()) {
        CommandDefinition cdef = (CommandDefinition)cli.next();
        if (command.equals(cdef.getName())) {
          return cdef;
        }
      }
      
    } catch (Exception e) {
      throw new PageGenerationException(e);
    }
    
    return null;
  }
  
  /**
   * Returns <CODE>true</CODE> if current page has andy command definition, <CODE>false</CODE> otherwise.
   */
  public boolean hasCommandDefinitions() throws PageGenerationException {
    return pageInfo.getPageConfig().getCommandDefinitions() != null;
  }
  
  private String variant = null;
  /**
   * Returns the name of the variant if current page is a variant page, <CODE>null</CODE> otherwise.
   */
  public String getVariant() {
    if (variant == null) {
      variant = request.getPlainParameter("_variant");
      if (variant == null) {
        variant = FileUtils.getCurrentPageVariant(request);
      } else {
        variant = variant.trim();
        FileUtils.setCurrentPageVariant(request, variant);
      }
    }
    return variant;
  }
  
  /**
   * Returns the Part of the current URL relative to the Context URL.
   * i.e. if the URL is <CODE>http://www.xyz.com/contextA/<B>showlist.doms?id=12</B></CODE> then <BR/>
   * <CODE>showlist.doms?id=12</CODE> is returned
   */
  public String getContextRelativeURL() {
    String absoluteContextPath = request.getContext().getRealPath("/");
    String absolutePagePath = pageInfo.getPageFile().getAbsolutePath();
    int len = absoluteContextPath.length();
    char c = absoluteContextPath.charAt(len-1);
    if (c == '\\' || c == '/') len--;
    String result = absolutePagePath.substring(len);
    return result.replace('\\','/');
  }
  
  /**
   * Returns the Part of the current URL relative to the Context URL.
   * i.e. if the URL is <CODE>http://www.xyz.com/<B>contextA/showlist.doms?id=12</B></CODE> then <BR/>
   * <CODE>contextA/showlist.doms?id=12</CODE> is returned
   */
  public String getHostRelativeURL() {
    return request.getContextPath() + getContextRelativeURL();
  }
  
  /**
   * Inserts the given page name into the "Pages-To-Be-Displayed-Next" list
   */
  public void insertNextPage(String pageName) {
    if (pageName == null || pageName.length() == 0) return;
    
    if (nextPages.length() == 0) {
      nextPages = pageName;
    } else {
      nextPages = pageName + "," + nextPages;
    }
  }
  
  /**
   * Inserts this page into the "Pages-To-Be-Displayed-Next" list
   */
  public void pushSelfName() {
    if (nextPages.length() == 0) {
      nextPages = page;
    } else {
      nextPages = page + "," + nextPages;
    }
  }
  
  /**
   * Saves "Pages-To-Be-Displayed-Next" list into the "nextpage" ADOM
   */
  public void flushNextPages() {
    ADOM adom = request.getRequestADOM("nextpage");
    if (adom == null) adom = request.newRequestADOM("nextpage");
    adom.setNodeValue("page", nextPages);
  }
  
  /**
   * Returns the PageInfo object attached to the current page
   */
  public PageInfo getPageInfo() throws PageGenerationException {
    if (pageInfo == null) {
      pageInfo = PageManager.getPageInfo(request,page,getVariant());
    }
    return pageInfo;
  }
 
}
