/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * NodeSelector.java
 *
 * Created on 13 July 2006 Wednesday, 00:23
 */

package org.moremotion.page;

import org.moremotion.adom.ADOMNode;
import org.moremotion.config.ConditionalListItem;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.servlet.SystemConfig;
import org.moremotion.servlet.SystemContext;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: NodeSelector.java 165 2008-07-01 15:29:42Z erkan $
 */
public class NodeSelector {
  
  private boolean selectAll;
  private ConditionalListItem[] nodelist;
  private MoreMotionRequest request;
  
  
  /**
   * Creates a new instance of NodeSelector
   */
  protected NodeSelector(boolean selectAll, ConditionalListItem[] nodelist, MoreMotionRequest request) {
    this.selectAll = selectAll;
    this.nodelist = nodelist;
    this.request = request;
  }
  
  /**
   * Checks the given ADOMNode agains the nodelist of the PageDataRequest and
   * performs the following actions:
   * <ol>
   *  <li>If the node name is in the list and "encrypt" is specified, encrypts the node and returns it. If "newName"
   * is also specified returned node will have the defined new name</li>
   * <li>If the node name is in the list and "newName" is specified, a new node is returned with the new name</li>
   * <li>If the node name is in the list, the given node is returned as it is</li>
   * <li>If "selectAll" is specified in the PageDataRequest then the given node is returned as it is</li>
   * <li>otherwise null is returned </li>
   * </ol>
   *
   */
  public ADOMNode processNode(ADOMNode adomNode) throws ConfigParameterException {
    
    if (nodelist == null) {
      return adomNode;
    }
    
    String nodeName = adomNode.getName();
    
    for (int i = 0; i < nodelist.length; i++) {
      
      DataRequestNode drn = (DataRequestNode)nodelist[i];
      if (nodeName.equals( drn.getName() )) {
        
        String newName = drn.getNewName(request);
        
        String key = drn.getEncryptionKey(request);
        if (key != null) {
          if (newName == null) newName = adomNode.getName();
          ADOMNode newAdomNode = new ADOMNode( newName );
          key = key + SystemConfig.requestParametersEncryptionKey(); 
          newAdomNode.setValue( SystemContext.encryptor().encrypt(adomNode.getValue(), key) ); 
          return newAdomNode;
          
        } else {
          if (newName == null) {
            return adomNode;
          }
          
          ADOMNode newAdomNode = new ADOMNode(newName);
          newAdomNode.setValue(adomNode.getValue());
          return newAdomNode;
        }
      }
    }
    
    if (selectAll) {
      return adomNode;
    }
    
    return null;
  }
  
  
}
