/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * PageDataRequest.java
 *
 */

package org.moremotion.page;

import org.moremotion.config.ConditionalList;
import org.moremotion.config.ConditionalListItem;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.config.ConfigurationException;
import org.moremotion.servlet.MoreMotionRequest;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Represents a single data request for a page.
 *
 * @author  Yusuf Boyacigil
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: PageDataRequest.java 51 2008-03-10 11:46:54Z erkan $
 */
public class PageDataRequest extends ConditionalListItem {
  
  private PageConfig pageConfig = null;
  private String name;
  private ConfigParameter sourceAreas;
  private ConfigParameter extraItemsBefore = null;
  private ConfigParameter extraItemsAfter = null;
  private ConfigParameter dataSource = null;
  private ConfigParameter selectAll = null;
  private ConditionalList nodes = null;
  
  protected PageDataRequest(PageConfig pageConfig, Element elm) throws ConfigurationException {
    
    super(elm);
    
    this.pageConfig = pageConfig;
    this.name = elm.getAttribute("name");
    
    String s;
    
    s = elm.getAttribute("selectAll");
    if (s.length() > 0) {
      this.selectAll = new ConfigParameter("selectAll",s);
    }
    
    s = elm.getAttribute("dataSource");
    if (s.length() > 0) {
      this.dataSource = new ConfigParameter("dataSource",s);
    }
    
    s = elm.getAttribute("sourceAreas");
    if (s.length() > 0) {
      this.sourceAreas = new ConfigParameter("sourceAreas",s);
    }
    
    s = elm.getAttribute("extraItemsBefore");
    if (s.length() > 0) {
      this.extraItemsBefore = new ConfigParameter("extraItemsBefore",s);
    }
    
    s = elm.getAttribute("extraItemsAfter");
    if (s.length() > 0) {
      this.extraItemsAfter = new ConfigParameter("extraItemsAfter",s);
    }
    
    parseNodeList(elm);
  }
  
  public PageDataRequest(String datasrc) {
    super(null);
    this.name = datasrc;
  }
  
  /**
   * Returns the PageConfig that this Data Request resides in
   */
  public PageConfig getPageConfig() {
    return this.pageConfig;  
  }
  
  
  /**
   * Returuns the name of this Data Request
   */
  public String getName() {
    return this.name;
  }
  
  /**
   * Returns <CODE>true</CODE> if this data request has no "node" child elements or the "selectAll" 
   * attribute is true.
   */
  public boolean getSelectAll(MoreMotionRequest request) throws ConfigParameterException {
    
    if (nodes == null) {
      return true;
    }
    return selectAll == null ? false : selectAll.resolve(request).booleanValue(false);
    
  }
  
  /**
   * Returns the name of the data source. If "dataSource" attribute is specified; its value, otherwise
   * the value of the "name" attribute is returned.
   */ 
  public String getDataSourceName(MoreMotionRequest request) throws ConfigParameterException {
    return this.dataSource == null ? this.name : this.dataSource.resolve(request).stringValue(this.name);
  }
  
  public String getSourceAreaNames(MoreMotionRequest request) throws ConfigParameterException {
    return this.sourceAreas == null ? null : this.sourceAreas.resolve(request).stringValue(null);
  }
  
  public int getExtraItemsBefore(MoreMotionRequest request) throws ConfigParameterException {
    return this.extraItemsBefore == null ? 0 : this.extraItemsBefore.resolve(request).intValue(0);
  }
  
  public int getExtraItemsAfter(MoreMotionRequest request) throws ConfigParameterException {
    return this.extraItemsAfter == null ? 0 : this.extraItemsAfter.resolve(request).intValue(0);
  }
  
  private void parseNodeList(Element elm) {
    
    NodeList nlist = elm.getElementsByTagName("node");
    if (nlist.getLength() == 0) {
      return;
    }
    
    nodes = new ConditionalList();
    
    for (int i = 0; i < nlist.getLength(); i++) {
      nodes.add( new DataRequestNode( (Element)nlist.item(i) ) );
    }

  }
  
  /**
   * Returns the node selector of the data request
   */
  public NodeSelector getNodeSelector(MoreMotionRequest request) throws ConfigParameterException {
    
    if (nodes == null) return null;
    
    ConditionalListItem[] nodelist = nodes.getItems(request);
    
    return new NodeSelector( getSelectAll(request), nodelist, request );
  }
  
  
  
  public String toString() {
    StringBuffer buffer = new StringBuffer();
    buffer.append("PageDatarequest [ name: " + this.name);
    buffer.append(", dataSource: " + (dataSource == null ? "" : dataSource.stringValue("") ) );
    buffer.append(", selectAll: " + (selectAll == null ? "" : selectAll.stringValue("") ) );
    buffer.append(", nodes: " +  (nodes == null ? "null" : "has nodes") );
    buffer.append(" ]");
    return buffer.toString();
  }
  
}