/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * TemplateCall.java
 *
 */

package org.moremotion.page;

import java.util.HashMap;
import org.moremotion.config.ConditionalListItem;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.servlet.MoreMotionRequest;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: TemplateCall.java 51 2008-03-10 11:46:54Z erkan $
 */
public class TemplateCall extends ConditionalListItem {
  
  private String name;
  private String originPage;
  private ConfigParameter beforeDS;
  private ConfigParameter errorPage;
  private ConfigParameter continuePage;
  private ConfigParameter debugLevel;
  private HashMap params;
      
  
  /** Creates a new instance of TemplateCall */
  protected TemplateCall(Element elm, String pageName) {
    super(elm);
    this.originPage = pageName;
    parse(elm);
  }
  
  private void parse(Element elm) {
    
    name = elm.getAttribute("name");
    beforeDS = new ConfigParameter("beforeDS", elm.getAttribute("beforeDS"));
    errorPage = new ConfigParameter("errorPage", elm.getAttribute("errorPage"));
    continuePage = new ConfigParameter("continuePage", elm.getAttribute("continuePage"));
    debugLevel = new ConfigParameter("debugLevel", elm.getAttribute("debugLevel"));
    
    NodeList nlist = elm.getElementsByTagName("param");
    if (nlist.getLength() == 0) {
      return;
    }
    
    params = new HashMap();
    for (int i = 0; i < nlist.getLength(); i++) {
      Element node = (Element) nlist.item(i);
      params.put( node.getAttribute("name"), node.getNodeValue() );
    }

  }  
  
  public String getName() {
    return name;
  }
  
  public boolean isBeforeDS(MoreMotionRequest request) throws ConfigParameterException {
    return beforeDS.resolve(request).booleanValue(false);
  }
  
  public String getErrorPage(MoreMotionRequest request) throws ConfigParameterException {
    return errorPage.resolve(request).stringValue(null);
  }
  
  public String getErrorPage() throws ConfigParameterException {
    return errorPage.stringValue(null);
  }
  
  public String getOriginPage() throws ConfigParameterException {
    return this.originPage;
  }
  
  public String getContinuePage() throws ConfigParameterException {
    return continuePage.stringValue(null);
  }
  
  public int getDebugLevel(MoreMotionRequest request) throws ConfigParameterException {
    return debugLevel.resolve(request).intValue(0);
  }
  
  public HashMap getParams() {
    return params;
  }
  
}
