/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * CallInstance.java
 *
 * Created on 23 April 2007 Monday, 01:54
 *
 */

package org.moremotion.process;

import org.moremotion.evaluator.Expression;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.Logger;
import org.moremotion.config.Config;
import org.moremotion.config.ConfigParameterException;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: CallInstance.java 162 2008-06-30 16:03:12Z erkan $
 */
public class CallInstance {
  
  protected boolean prepared = false;
  protected CallInstance parent = null;
  protected CallDefinition callDef = null;
  protected Process process = null;
  protected CallInstance[] children = null;
  protected ProcessBlock processBlock = null;
  protected ProcessRecord processRecord = null;
  protected Expression recordFilter = null;
  protected String blockName = null;
  protected Logger logger = null;
  protected Exception exception = null; 
  protected int completionCode = 0;
  protected int level = -1;
  protected int resetDepth = 200;
  protected Config config = null;
  
  public CallInstance(CallInstance parent, CallDefinition callDef, MoreMotionRequest request) throws ConfigParameterException  {
    this.parent = parent;
    this.callDef = callDef;
    setLevel();
    setReset(request);
  }
  
  public Logger getLogger(Logger baseLogger) {
    if (logger == null) {
      logger = new Logger(baseLogger.getId());
      logger.setResourceBundleName(baseLogger.getResourceBundleName());
      logger.setIndent(level-1);
    }
    return logger;
  }
  
  private void setLevel() {
    level = -1;
    CallInstance ci = this;
    while (ci != null) {
      level++;
      ci = ci.parent;
    }
  }
  
  private void setReset(MoreMotionRequest request) throws ConfigParameterException { 
    if (this.callDef == null) return;
    String value = this.callDef.getReset(request);
    if (value == null) {
      if (this.parent == null || this.parent.parent == null) {
        // The default of the first level callDefinitions is "true"
        resetDepth = 0;
      }
      return;
    }
    if (value.equalsIgnoreCase("true")) {
      resetDepth = 0;
    }
    else if (value.equalsIgnoreCase("false")) {
      resetDepth = 88;
    }
    else {
      boolean first = true;
      for (int i = 0; i < value.length(); i++) {
        char c = value.charAt(i);
        if (c == '.') {
          if (first) {
            resetDepth = -1;
            first = false;
          }
          resetDepth++;
        } else {
          throw new ConfigParameterException("INVALID_RESET_VALUE",new String[]{callDef.getTargetName(), value},null);
        }
      }
    }
  }
  
  public void resetCompletionCode(int cc) {
    this.completionCode = cc;
    if (parent != null) {
      parent.resetCompletionCode(cc);
    }
  }
  
  public void setCompletionCode(int cc) {
    if (completionCode < cc) {
      completionCode = cc;
      if (parent != null) {
        parent.setCompletionCode(cc);
      }
    }
  }
  
  public int getCompletionCode() {
    return this.completionCode;
  }
  
  public int getMaxCompletionCode() {
    if (parent != null) {
      return parent.getMaxCompletionCode();
    } else {
      return completionCode;
    }
  }
  
  public int getMaxLevelCompletionCode() {
    if (parent != null) {
      return parent.completionCode;
    } else {
      return completionCode;
    }
  }
  
  public String toString() {
    
    StringBuffer buf = new StringBuffer();
    
    buf.append( "callDef : " + callDef.toString() );
    
    return buf.toString();
  }
  
  
}