/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * FormProcessBlock.java
 *
 * Created on October 16, 2003, 7:47 PM
 */

package org.moremotion.process;

import java.util.HashMap;
import java.util.Iterator;
import java.util.ArrayList;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.StringParser;

/**
 * Represents a Process Block that includes one or more {@link org.moremotion.process.ProcessRecord Process Records}
 *
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: FormProcessBlock.java 51 2008-03-10 11:46:54Z erkan $
 */
public class FormProcessBlock implements ProcessBlock {
  
  private MoreMotionRequest request;
  private BlockInfo pbinfo;
  
  private HashMap formFields;
  private HashMap customFields;
  private HashMap objects;
  private RecordStatus[] recstats = null;
  private int recordCount;
  
  /**
   * Creates new FormProcessBlock.
   */
  protected FormProcessBlock(MoreMotionRequest request, String blockName, String pbinfoStr) throws ProcessManagingException {
    this.request = request;
    formFields = new HashMap();
    pbinfo = new BlockInfo(blockName, pbinfoStr);
    prepareNormalFields(request);
    prepareCheckBoxes(request);
  }
  
  /**
   * Clears all the process records of this process block.
   */
  public void clearRecords() {
    formFields.clear();
  }
  
  /**
   * Returns the names of the block fields.
   */
  private String[] _fieldNames = null;
  public String[] getFieldNames() {
    if (_fieldNames == null) {
      Iterator itr = formFields.keySet().iterator();
      ArrayList a = new ArrayList();
      while (itr.hasNext()) {
        a.add((String)itr.next());
      }
      _fieldNames = new String[a.size()];
      a.toArray(_fieldNames);
    }
    
    return _fieldNames;
  }
  
  public String getField(String fieldName, int index) {
    
    if (fieldName == null) return null;
    
    if (fieldName.equals("_selected")) {
      if (recstats == null) return "false";
      return recstats[index].isSelected() ? "true" : "false";
      
    } else if(fieldName.equals("_modified")) {
      if (recstats == null) return "false";
      return recstats[index].isModified() ? "true" : "false";
      
    } else if(fieldName.equals("_modified_fields")) {
      if (recstats == null) return "";
      return recstats[index].getModifiedFieldsAsString();
      
    } else if(fieldName.equals("_parent_recnum")) {
      if (recstats == null) return "";
      return recstats[index].getParentRecNum();
    }
    
    String result = null;
    
    if (customFields != null) {
      String key = fieldName + " " + index;
      result = (String)customFields.get(key);
    }
    
    if (result == null && formFields != null) {
      String[] record = (String[])formFields.get(fieldName);
      if (record != null) {
        return record[index];
      }
    }
    
    return result;
  }
  
  public boolean isModified(int index) {
    if (recstats == null) return false;
    return recstats[index].isModified();
  }
  
  public boolean isSelected(int index) {
    if (recstats == null) return false;
    return recstats[index].isSelected();
  }
  
  public boolean isFieldModified(String fieldName, int index) {
    if (recstats == null) return false;
    return recstats[index].isFieldModified(fieldName);
  }
  
  public RecordStatus getRecordStatus(int index) {
    return this.recstats[index];  
  }
  
  public int getRecordCount() {
    return recordCount;
  }
  
  public void setField(String fieldName, String value, int index) {
    String key = fieldName + " " + index; 
    if (customFields == null) customFields = new HashMap();
    customFields.put(key, value);
  }
  
  public Object getObject(String fieldName, int index) {
    if (objects == null) return null;
    String key = fieldName + " " + index;
    return objects.get(key);
  }
  
  public void setObject(String fieldName, Object object, int index) {
    String key = fieldName + " " + index;
    if (objects == null) objects = new HashMap();
    objects.put(key,object);
  }
  
  
  private boolean isCheckBox(String name) {
    if (pbinfo.cboxNames == null) return false;
    
    for (int i = 0; i < pbinfo.cboxNames.length; i++) {
      String cboxName = pbinfo.cboxNames[i];
      if (name.equals(cboxName)) {
        return true;
      }
    }
    return false;
  }
  
  private void prepareNormalFields(MoreMotionRequest request) {
    
    String statusFieldName = "px_" + pbinfo.sequence + "_recstat";
    String[] statuses = request.getParameterValues(statusFieldName);
    
    if (statuses != null) {
      
      recordCount = statuses.length;
      
      recstats = new RecordStatus[recordCount];
      
      for (int i = 0; i < recordCount; i++) {
        recstats[i] = new RecordStatus(statuses[i]);  
      }
      
      String prefix = "pf_" + pbinfo.sequence + "_";
      
      if (pbinfo.fieldNames != null) {
        for (int i = 0; i < pbinfo.fieldNames.length; i++) {
          String fieldName = pbinfo.fieldNames[i];
          if (!isCheckBox(fieldName)) {
            String[] v = request.getParameterValues(prefix + fieldName);
            formFields.put(fieldName,v);
          }
        }
      }
      
    }
    
  }
  
 
  private void prepareCheckBoxes(MoreMotionRequest request) {
    
    if (pbinfo.cboxNames == null) return;
    
    for (int i = 0; i < pbinfo.cboxNames.length; i++) {
      String cboxName = pbinfo.cboxNames[i];
      
      String[] cboxArray = new String[recordCount];
      for (int j = 0; j < recordCount; j++) {
        cboxArray[j] = "false";
      }
      String[] valueArray = request.getParameterValues("pf_" + pbinfo.sequence + "_" + cboxName);
      if (valueArray != null) {
        for (int k = 0; k < valueArray.length; k++) {
          try {
            int ix = Integer.parseInt(valueArray[k]);
            cboxArray[ix] = "true";
          } catch (Exception e) {}
        }
      }
      
      formFields.put(cboxName,cboxArray);
    }
    
  }
  
  public String toString() {
    StringBuffer buf = new StringBuffer();
    buf.append("Form PB");
    buf.append("; record Q: " + recordCount );
    buf.append("; field Names : [");
    if (pbinfo != null && pbinfo.fieldNames != null) {
      for (int i = 0; i < pbinfo.fieldNames.length; i++) {
        if (i > 0) buf.append(',');
        buf.append(pbinfo.fieldNames[i]);
      }
    }
    buf.append("]");
    
    return buf.toString();
  }
  
  
}

class BlockInfo {
  
  protected String name;
  protected String sequence;
  protected String[] fieldNames = null;
  protected String[] cboxNames = null;
  
  /** Creates a new instance of BlockInfo */
  protected BlockInfo(String name, String pbinfoStr) throws ProcessManagingException {
    
    this.name = name;
    
    int x = pbinfoStr.indexOf(';');
    if (x == -1) {
      throw new ProcessManagingException("PBINFO_INVALID",new String[]{name, pbinfoStr},null);
    }
    sequence = pbinfoStr.substring(0,x);
    StringParser sp = new StringParser(pbinfoStr.substring(x+1),',');
    
    ArrayList a0 = null;
    ArrayList a1 = null;
    
    while (sp.hasMore()) {
      String def = sp.nextItem();
      String fieldName;
      char type;
      // int k = def.indexOf(':');
      
      if (def.length() > 2 && def.charAt(1) == ':') {
        type = def.charAt(0);
        fieldName = def.substring(2);
      } else {
        type = ' ';
        fieldName = def;
      }
      
      if (a0 == null) a0 = new ArrayList();
      a0.add(fieldName);
      
      if (type == 'c') {
        if (a1 == null) a1 = new ArrayList();
        a1.add(fieldName);
        
      }
      
    }
    
    if (a0 != null) {
      fieldNames = new String[a0.size()];
      a0.toArray(fieldNames);
    }
    
    if (a1 != null) {
      cboxNames = new String[a1.size()];
      a1.toArray(cboxNames);
    }
    
    
  }
  
  
  
  
}