/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * ResourceBundle.java
 *
 */

package org.moremotion.resource;

import javax.servlet.ServletContext;
import java.util.Properties;
import java.text.MessageFormat;
import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMManager;
import org.moremotion.adom.ADOMNode;

/** Represents a Resource Bundle that contains resources in one single language.
 * The resource bundles are loaded by the resource manager from resource files 
 * that have ".res" extension under the "/WEB-INF/MM-INF/resources" folder.
 * 
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ResourceBundle.java 51 2008-03-10 11:46:54Z erkan $
 */
public class ResourceBundle {
  
  private String name = null;
  private Properties source = null;
  private long lastModified;
  private ADOM resadom = null;
  private boolean dsres = false; 

  
  
  /** 
   * Creates a ResourceBundle
   * @param source Properties object that contains the resource ids and resource messages.
   * @param lastModified Last time the resource files is modified.
   */  
  public ResourceBundle(Properties source, long lastModified) {
    this.source = source;
    this.lastModified = lastModified;
  }
  
  
  /** 
   * Creates a ResourceBundle
   * @param dsname the name of the datasource that keeps the resources
   */  
  public ResourceBundle(String dsname, ServletContext context, String language) {
    this.dsres = true;
    this.resadom = ADOMManager.getApplicationADOM(context, dsname + "_" + language);
  }
  
  
  protected Properties getResources() {
    return this.source;
  }
  
  /**
   * Sets the name of this resource bundle.
   */
  public void setName(String name) {
    this.name = name;
  }
  
  /**
   * Returns the name of this resource bundle.
   */
  public String getName() {
    return this.name;  
  }
  
  
  
  private String getResourceFromDataSource(String resourceId) {
    if (resadom == null) return "";
    ADOMNode node = resadom.findNode(resourceId);
    return node == null ? "" : node.getValue();
  }
  
  /**
   * Returns the text of the specified resource.
   */
  public String getResourceText(String resourceId) {
    
    if (dsres) {
      return getResourceFromDataSource(resourceId);
    }
    
    String result = source.getProperty(resourceId);
    if (result == null) {
      ResourceBundle defaultBundle;
      try {
        defaultBundle = ResourceManager.getResourceBundle(this.name,"en");
        return defaultBundle._getResourceText(resourceId);
      } catch (ResourceException ex) {}
    }
    
    return result == null ? resourceId : result;
  }
  
  private String _getResourceText(String resourceId) {
    String result = source.getProperty(resourceId);
    return result == null ? resourceId : result;
  }
  
  /** 
   * Returns whether or not the resource file is modified.
   * @param lastModified The last modified date of the resource file.
   * @return <CODE>true</CODE> if modified, <CODE>false</CODE> otherwise.
   */  
  public boolean isModified(long lastModified) {
    return (lastModified > this.lastModified);
  }
  
  public static void  main(String[] args) throws Exception {
  }
  
}