/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * AccessControlConfigParameter.java
 *
 */

package org.moremotion.security;

import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.servlet.MoreMotionRequest;
import org.w3c.dom.Element;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: AccessControlConfigParameter.java 51 2008-03-10 11:46:54Z erkan $
 */
public class AccessControlConfigParameter {
  
  private ConfigParameter secdom = null;
  private ConfigParameter checkPoint = null;
  private ConfigParameter loginPage = null;
  private ConfigParameter failurePage = null;
  
  /** Creates a new instance of AccessControlConfigParameter */
  public AccessControlConfigParameter(Element elm) throws ConfigParameterException {
    parse(elm);
  }
  
  public void parse(Element elm) throws ConfigParameterException {
    
    String s = elm.getAttribute("securityDomain");
    if (s.length() > 0) {
      secdom = new ConfigParameter("securityDomain",s);
    }
    
    s = elm.getAttribute("checkPoint");
    if (s.length() > 0) {
      checkPoint = new ConfigParameter("checkPoint",s);
    }
    
    s = elm.getAttribute("loginPage");
    if (s.length() > 0) {
      loginPage = new ConfigParameter("loginPage",s);
    }
    
    s = elm.getAttribute("failurePage");
    if (s.length() > 0) {
      failurePage = new ConfigParameter("failurePage",s);
    }
    
  }    
  
  public AccessControlDefinition getAccessControlDefinition(MoreMotionRequest request) throws ConfigParameterException {
    
    return new AccessControlDefinition(
        secdom == null ? null : secdom.resolve(request).stringValue(null),
        checkPoint == null ? null : checkPoint.resolve(request).stringValue(null),
        loginPage == null ? null : loginPage.resolve(request).stringValue(null),
        failurePage == null ? null : failurePage.resolve(request).stringValue(null)
        );
    
  }
  
}
