/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * Encrptor.java
 *
 * Created on 17 January 2007 Wednesday, 23:02
 */

package org.moremotion.security;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: Encryptor.java 51 2008-03-10 11:46:54Z erkan $
 */
public interface Encryptor {
  
  /**
   * Encrypts the given string in to a non-decryptable string. This method is typically used for
   * password encryptions.
   * @param str The string to encrypt.
   * @return Encrypted string which can not be decrypted.
   */
  public String oneWayEncrypt(String str);
  
  /**
   * Encrypts the given string in to a decryptable string.
   * @param str The string to encrypt.
   * @param key The encryption key to be used during encryption.
   * @return Encrypted string which can be decrypted.
   */
  public String encrypt(String str, String key);
  
  /**
   * Decrypts the given string.
   * @param str The encrypted string to decrypt.
   * @param key The encryption key used during encryption.
   * @return The decrypted string. 
   * @throws DecryptionException if decrytion fails
   */ 
  public String decrypt(String str, String key) throws DecryptionException;   
  
  
  /**
   * Checks if the given string is encrypted.
   * @param str The string to check
   * @param key The encryption key used during encryption.
   * @return <CODE>true</CODE> if the given string is an encrypted string <CODE>false</CODE> otherwise
   */ 
  public boolean isEncrypted(String str, String key);
  
}
