/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.servlet;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.moremotion.action.ActionServiceManager;
import org.moremotion.process.ProcessManager;
import org.moremotion.util.JdbcConnection;
import org.moremotion.util.XString;

/**
 * Service Invoker Servlet loads MoreMotion Services that are mapped to URL patterns. If an URL contains
 * '.doms' extension i.e. <CODE>mypack.MyService.doms?parm1=value1</CODE> this servlet loads
 * <CODE>mypack.MyService</CODE> class and calls their <CODE>doService()</CODE> methods.
 *
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ServiceInvokerServlet.java 200 2008-09-09 19:07:21Z erkan $
 */

public class ServiceInvokerServlet extends HttpServlet {
  
  private void processRequest(MoreMotionRequest mmrequest) 
  throws ServletException, java.io.IOException {
    
    try {
      
      String url = XString.afterLast(mmrequest.getRequestURI(),'/');
      
      if (url.endsWith(".mmpage")) {
        String page = mmrequest.getPlainParameter("pg");
        if (page == null || page.trim().length() == 0) {
          page = XString.beforeLast(url,'.').replace('.','/');
        }
        mmrequest.generatePage(page);
        return;
      }
      
      if (url.equals("") || url.equals("index.htm")) {
        mmrequest.generatePage("/index");
        return;
      }
      
      String serviceName = XString.beforeLast(url,'.');
      
      if (serviceName.equals("display")) {
        String page = mmrequest.getPlainParameter("pg");
        if (page == null || page.equals("/")) { 
          page = "/index";
        }  
        mmrequest.generatePage(page);
        return;
      }
      
      if (serviceName.equals("ProcessManager")) {
        ProcessManager.doService(mmrequest);
        return;
      }
      
      ActionServiceManager.runActionService(serviceName, mmrequest );
      
    } catch(Exception mme) {
      try {
        mmrequest.createMessage(mme);
        mmrequest.generateDefaultErrorPage();
      } catch (Exception e) {
        throw new ServletException(e);
      }
    }
    
  }
  
  private void _processRequest(HttpServletRequest request, HttpServletResponse response) 
  throws ServletException, java.io.IOException {
    
    MoreMotionRequest mmrequest = null;
    try {
      mmrequest = new MoreMotionRequest(request, response);
      processRequest(mmrequest);
      
    } finally {
      try {
        if (mmrequest != null) {
          JdbcConnection.releaseRequestConnections(mmrequest);
        }
      } catch (Exception e) {
        throw new ServletException(e);    
      }
    }
    
  }
  
  
  protected void doGet(HttpServletRequest request, HttpServletResponse response)
  throws ServletException, java.io.IOException {
    
    _processRequest(request, response);
    
  }
  
  protected void doPost(HttpServletRequest request, HttpServletResponse response)
  throws ServletException, java.io.IOException {
    
    _processRequest(request, response);
    
  }
  
  public String getServletInfo() {
    return "MoreMotion Service Invoker Servlet";
  }
  
  private static long lModified=0;
  
  protected long getLastModified(HttpServletRequest req) {
    return lModified++;
  }

}