/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * StartupServlet.java
 *
 */

package org.moremotion.servlet;

import java.io.File;
import java.util.Properties;
import javax.naming.InitialContext;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.moremotion.cache.FileChangeDetector;
import org.moremotion.cache.FileChangeNotification;
import org.moremotion.cache.TempFileManager;
import org.moremotion.cache.XslTransformerCache;
import org.moremotion.cache.ZipDeployer;
import org.moremotion.config.ConfigManager;
import org.moremotion.config.ConfigUnit;
import org.moremotion.page.UserAgentAnalyzer;
import org.moremotion.util.Logger;
import org.moremotion.security.Encryptor;
import org.moremotion.util.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 * MoreMotion Startup Servlet. It initializes system level variables and starts MoreMotion managing services.
 *
 * @author Yusuf Boyacigil
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: StartupServlet.java 138 2008-05-11 17:25:44Z erkan $
 */
public class StartupServlet extends HttpServlet implements FileChangeNotification {
  
  protected static String mmHomePath = null;
  protected static File mmHomeFile;
  protected static ServletContext servletContext;
  protected static InitialContext initialContext = null;
  
  private File jndiProps;
  
  private static String contextPath;
  
  protected static ConfigManager configManager;
  protected static TempFileManager tempFileManager;
  protected static TempFileManager tempWebFileManager;
  protected static ZipDeployer zipDeployer;
  protected static XslTransformerCache transformerCache;
  
  protected static String serviceExtension = "doms";
  protected static boolean iSymbolFirst = false;
  protected static boolean disableDebug = false;
  protected static boolean enableParameterConversion = true;
  protected static String serverEncoding = "iso-8859-1";
  protected static File applRootFile = null;
  
  protected static Encryptor encryptor = null;
  
  protected static Logger logger = null;
  
  
  /** Creates a new instance of StartupServlet */
  public StartupServlet() {
  }
  
  public void init(ServletConfig config) throws ServletException {
    
    super.init(config);
    
    servletContext = getServletContext();
    
    mmHomePath = getServletContext().getRealPath("/WEB-INF/MM-INF");
    mmHomeFile = new File(mmHomePath);
    applRootFile = mmHomeFile.getParentFile().getParentFile();
    
    loadSystemParameters();

    logger = new Logger("MoreMotion",1);
    logger.setResourceBundleName("moremotion");
    logger.setIdLength(10);
    logger.printres("MOREMOTION_STARTED", getServletContext().getRealPath("/") );
    
    
    jndiProps = new File(mmHomeFile, "jndi.properties");
    initInitialContext(jndiProps);
    
    transformerCache = new XslTransformerCache(10);
    
    /*
     * Start File Change Detector
     */
    FileChangeDetector.getInstance();
    
    /* Register jndiProps file for change detection */
    FileChangeDetector.getInstance().addFile(jndiProps,this);
    
    /*
     * Start Temporary File Managers
     */
    tempFileManager = new TempFileManager( new File(mmHomeFile,"/~tempfiles") );
    tempWebFileManager = new TempFileManager( new File(applRootFile, "~tempfiles") );
    
    /*
     * Start ZipDeployer
     */
    zipDeployer = new ZipDeployer();
    
    /*
     * Start ConfigManager
     */
    configManager = new ConfigManager( new File(mmHomeFile,"config") );
    
  }
  
  
  private void loadSystemParameters() {
    
    try {
      
      File f = new File(mmHomeFile, "config" + File.separator + "sysinfo.xml");
      
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      
      Document document = builder.parse(f);
      NodeList nlist = document.getElementsByTagName("root").item(0).getChildNodes();
      
      for (int i = 0; i < nlist.getLength(); i++) {
        
        Node node = nlist.item(i);
        
        if (node.getNodeType() == Node.ELEMENT_NODE) {
          String tagName = node.getNodeName();
          String name = ((Element)node).getAttribute("name");
          ConfigUnit unit = new ConfigUnit((Element) node, "*");
          
          if (tagName.equals("systemParameters")) {
            SystemConfig.parse( unit.getConfigNodeList() );
          }
        }
      }
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }
  
  private void initInitialContext(File jndiProps) {
    
    InitialContext ic = null;
    
    try {
      
      ic = (InitialContext)getServletContext().getAttribute("JDNI.Naming.InitialContext");
      
      if (ic == null || !(ic instanceof InitialContext) ) {
        Properties props = null;
        if (jndiProps.exists()) {
          props = new Properties();
          props.load( new java.io.FileInputStream(jndiProps) );
          logger.printres("LOADING_JNDI_PROPERTIES", jndiProps.getAbsolutePath());
        }
        
        if (props == null) {
          ic = new InitialContext();
        } else {
          ic = new InitialContext(props);
        }
        
        getServletContext().setAttribute("JDNI.Naming.InitialContext", ic);
      } else {
        logger.printres("INITIAL_CONTEXT_ATTRIBUTE_PRESENT","");
      }
      
      initialContext = ic;
      
    } catch(Exception e) {
      logger.print("INITIAL_CONTEXT_ERROR","");
      e.printStackTrace();
    }
    
  }
  
  
  
  /*
   * This method called by SystemConfig after his refresh() method is called by the
   * ConfigManager.
   */
  public static void systemConfigChanged() {
    
    UserAgentAnalyzer.init();
    
    String s = SystemConfig.encryptorClassName();
    try {
      encryptor = (org.moremotion.security.Encryptor)Class.forName(s).newInstance();
    } catch (Exception e) {
      
      logger.printres("ENCRYPTOR_CLASS_LOAD_ERROR",s);
      e.printStackTrace();
    }
    
    transformerCache.setCapacity(SystemConfig.xsltTransformerCacheCapacity());
    
  }
  
  
  public void doChange(File retFile) {
    
    if (retFile == jndiProps) {
      initInitialContext(retFile);
    }
    
  }
  
  protected void doGet(HttpServletRequest request, HttpServletResponse response)
  throws ServletException, java.io.IOException {
    processRequest(request, response);
  }
  
  protected void doPost(HttpServletRequest request, HttpServletResponse response)
  throws ServletException, java.io.IOException {
    processRequest(request, response);
  }
  
  protected void processRequest(HttpServletRequest request, HttpServletResponse response)
  throws ServletException, java.io.IOException {
    
    response.setContentType("text/html");
    java.io.PrintWriter out = response.getWriter();
    out.println("<html>");
    out.println("<head>");
    out.println("<title>MoreMotion Startup Servlet</title>");
    out.println("</head>");
    out.println("<body>");
    out.println("Current MoreMotion home is: " + mmHomePath);
    out.println("</body>");
    out.println("</html>");
    out.close();
  }
  
  public void doDelete(File f) {
  }
  
  
  public void destroy() {
    FileChangeDetector.getInstance().stop();
    configManager.stop();
    tempFileManager.stop();
    tempWebFileManager.stop();
    zipDeployer.stop();
  }
  
  
}
