/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * SystemConfig.java
 *
 */

package org.moremotion.servlet;

import java.io.File;
import java.util.HashMap;
import org.moremotion.config.ConfigNode;
import org.moremotion.config.ConfigNodeList;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.config.ConfigUnit;
import org.moremotion.config.ConfigurationException;
import org.moremotion.util.Encryption;

/**
 * Initializes MoreMotion on application server start-up.
 *
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: SystemConfig.java 86 2008-03-21 19:21:24Z erkan $
 */
public class SystemConfig {

  private static String actionServiceExtension = "doms";
  private static String serverEncoding = "iso-8859-1";
  private static boolean iSymbolFirst = false;
  private static boolean disableDebug = false;
  private static boolean enableParameterConversion = true;

  private static String enabledUtilityServices = "";

  private static String encryptorClassName = "org.moremotion.security.DefaultEncryptor";
  private static String requestParametersEncryptionKey = "*";

  private static int xsltTransformerCacheCapacity = 10;
  private static int configurationScanIntervalInSeconds = 5;
  
  private static String applicationLanguage = "en";
  private static boolean generatePageInfo = false;

  private static HashMap params = new HashMap();


  public static String mmHomePath() {
    return StartupServlet.mmHomePath;
  }

  public static File mmHomeFile() {
    return StartupServlet.mmHomeFile;
  }

  public static File applRootFile() {
    return StartupServlet.applRootFile;
  }

  public static String actionServiceExtension() {
    return actionServiceExtension;
  }

  public static String serverEncoding() {
    return serverEncoding;
  }

  public static String enabledUtilityServices() {
    return enabledUtilityServices;
  }

  public static String encryptorClassName() {
    return encryptorClassName;
  }

  public static String requestParametersEncryptionKey() {
    return requestParametersEncryptionKey;
  }

  public static boolean iSymbolFirst() {
    return iSymbolFirst;
  }

  public static boolean disableDebug() {
    return disableDebug;
  }

  public static boolean enableParameterConversion() {
    return enableParameterConversion;
  }

  public static int xsltTransformerCacheCapacity() {
    return xsltTransformerCacheCapacity;
  }

  public static String getParameter(String name, String defValue) {
    String value = (String) params.get(name);
    return value == null || value.length() == 0 ? defValue : value;
  }

  public static int configurationScanIntervalInSeconds() {
    return configurationScanIntervalInSeconds;
  }
  
  public static String applicationLanguage() {
    return applicationLanguage;
  }

  public static boolean generatePageInfo() {
    return generatePageInfo;
  }


  public static void refresh(ConfigUnit unit, boolean master) throws ConfigurationException {

   if (master) {
     parse( unit.getConfigNodeList() );
     StartupServlet.systemConfigChanged();
   } else {
     parseSubConfig( unit.getConfigNodeList() );
   }

  }

  public static void parse(ConfigNodeList nodelist) throws ConfigParameterException {

    boolean transformerFactoryDefined = false; 
    
    for (int i = 0; i < nodelist.size(); i++) {

      ConfigNode node = nodelist.get(i);

      String name = node.getName();
      String value = node.getValue();
      String type = node.getElement().getAttribute("type");
      
      if (type.equals("password")) {
        if (Encryption.isEncrypted(value,10,52845,11719,133)) {
          value = Encryption.decrypt(value,10,52845,11719,133);
        }        
      }

      if (name.equals("actionServiceExtension")) {
        actionServiceExtension = value;
      }

      else if (name.equals("applicationLanguage")) {
        applicationLanguage = value;
      }

      else if (name.equals("serverEncoding")) {
        serverEncoding = value;
      }

      else if (name.equals("iSymbolFirst")) {
        iSymbolFirst = value.equalsIgnoreCase("true");
      }

      else if (name.equals("generatePageInfo")) {
        generatePageInfo = value.equalsIgnoreCase("true");
      }

      else if (name.equals("disableDebug")) {
        disableDebug = value.equalsIgnoreCase("true");
      }

      else if (name.equals("enableParameterConversion")) {
        enableParameterConversion = value.equalsIgnoreCase("true");
      }

      else if (name.equals("encryptorClassName")) {
        encryptorClassName = value;
      }

      else if (name.equals("requestParametersEncryptionKey")) {
        requestParametersEncryptionKey = value;
      }

      else if (name.equals("enabledUtilityServices")) {
        enabledUtilityServices = value;
      }

      else if (name.equals("xsltTransformerCacheCapacity")) {
        try {
          xsltTransformerCacheCapacity = Integer.parseInt(value);
        } catch (Exception e) {
          throw new ConfigParameterException("INVALID_CONFIGURATION_PARAMETER_VALUE",new String[]{name, value},null);
        }
      }

      else if (name.equals("configurationScanIntervalInSeconds")) {
        try {
          configurationScanIntervalInSeconds = Integer.parseInt(value);
        } catch (Exception e) {
          throw new ConfigParameterException("INVALID_CONFIGURATION_PARAMETER_VALUE",new String[]{name, value},null);
        }
      }
      
      else if (name.equals("javax.xml.transform.TransformerFactory")) {
        transformerFactoryDefined = true;    
      }

      /*
       * is a Java System Property setting?
       */
      if (type.equals("System.property")) {
        System.setProperty(name,value);
      }
      
      params.put(name, value) ;

    }
    
    if (!transformerFactoryDefined) {
      try {
        Class.forName("com.sun.org.apache.xalan.internal.xsltc.trax.TransformerFactoryImpl");
        System.setProperty("javax.xml.transform.TransformerFactory",
            "com.sun.org.apache.xalan.internal.xsltc.trax.TransformerFactoryImpl");
      } catch (Exception ex) {}
    }
    
    if (requestParametersEncryptionKey.length() == 0) {
      requestParametersEncryptionKey = "*";
    }
    
    if (applicationLanguage == null || applicationLanguage.trim().length() == 0) {
      applicationLanguage = "en";
    }
    
  }


  
  public static void parseSubConfig(ConfigNodeList nodelist) throws ConfigParameterException {

    for (int i = 0; i < nodelist.size(); i++) {

      ConfigNode node = nodelist.get(i);

      String name = node.getName();
      String value = node.getValue();
      String type = node.getElement().getAttribute("type");
      
      if (type.equals("password")) {
        if (Encryption.isEncrypted(value,10,52845,11719,133)) {
          value = Encryption.decrypt(value,10,52845,11719,133);
        }        
      }

      /*
       * is a Java System Property setting?
       */
      if (type.equals("System.property")) {
        System.setProperty(name,value);
      }
      
      params.put(name, value) ;
    }
    
  }


}