/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * SetADOM.java
 *
 * Created on July 8, 2001, 5:31 PM
 */

package org.moremotion.utilityservices;

import java.io.IOException;
import java.util.Enumeration;

import org.moremotion.action.ActionServiceContext;
import org.moremotion.adom.ADOM;
import org.moremotion.page.PageGenerationException;
import org.moremotion.action.ActionService;
import org.moremotion.servlet.ServiceException;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.adom.ADOMUtils;


/**
 * Sets value to a ADOMNode
 * @author  Yusuf Boyacigil
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: SetADOM.java 51 2008-03-10 11:46:54Z erkan $
 */
public class SetADOM implements ActionService {
  
  private static final String REQ_PARAM_NEXTPAGE = "nextpage";
  private static final String EMPTY_STRING = "";
  
  
  public void doService(ActionServiceContext asc) throws ServiceException, IOException {
    
    MoreMotionRequest request = asc.getRequest();
        
    if (!Utils.isUtilityServiceActive(request,"SetADOM")) return; 

    String firstAdomKey = null;
    
    Enumeration e = request.getParameterNames();
    
    while(e.hasMoreElements()) {
      
      String param = (String) e.nextElement();
      param = param.trim();
      
      if ( !param.equals("") && (param.indexOf("/") > -1 || param.indexOf(".") > -1) ) {
        
        String adomKey = ADOMUtils.getPureADOMName(param);
        
        if (firstAdomKey == null) {
          firstAdomKey = adomKey;
        }
        
        char scope = ' ';
        ADOM adom = ADOMUtils.findADOM(request, param);
        if (adom == null) {
          scope = ADOMUtils.getADOMScope(param);
          if (scope == 'r') adom = request.newRequestADOM(adomKey);
          else if (scope == 'a') adom = request.newApplicationADOM(adomKey);
          else adom = request.newSessionADOM(adomKey);
        }
        if (scope != 'a' || !adom.isProtected() ) {
          String nodePath = ADOMUtils.getNodePath(param);
          if (nodePath != null) {
            adom.setNodeValue(nodePath, request.getParameter(param, EMPTY_STRING));
          }
        }
      }
    }
    
    String nextPage = request.getParameter("nextpage");
    if (nextPage != null) {
      request.generatePage(nextPage);
      return;
    }
    request.getResponse().sendRedirect( "ViewADOM.doms?adom=" + firstAdomKey);
  }


}