/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * SessionListenerQueue.java
 *
 * Created on 24 September 2007 Monday, 00:01
 *
 */

package org.moremotion.session;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import javax.servlet.http.HttpSessionEvent;
import javax.servlet.http.HttpSessionListener;

/**
 * @author Erkan Cengiz
 * @version $Id: SessionListenerQueue.java 51 2008-03-10 11:46:54Z erkan $
 */
public class SessionListenerQueue  implements HttpSessionListener {
  
  private static HashMap listeners = new HashMap();
  
  public static void addListener(String className) throws ClassNotFoundException, InstantiationException, IllegalAccessException {
    if (listeners.get(className) == null) {
      HttpSessionListener listener = (HttpSessionListener) Class.forName(className).newInstance();
      listeners.put(className,listener);
    }
  }
  
  public static void addListener(HttpSessionListener listener) throws ClassNotFoundException, InstantiationException, IllegalAccessException {
    String className = listener.getClass().getName();
    if (listeners.get(className) == null) {
      listeners.put(className,listener);
    }
  }
  
  public static void removeListener(String className) {
    listeners.remove(className);
  }
  
  public static void removeListener(HttpSessionListener listener) {
    String className = listener.getClass().getName();
    listeners.remove(className);
  }
  
  public void sessionCreated(HttpSessionEvent se) {
    Iterator itr = listeners.entrySet().iterator();
    while (itr.hasNext()) {
      Map.Entry e = (Map.Entry) itr.next();
      HttpSessionListener listener = (HttpSessionListener) e.getValue();
      listener.sessionCreated(se);
    }
  }
  
  public void sessionDestroyed(HttpSessionEvent se) {
    Iterator itr = listeners.entrySet().iterator();
    while (itr.hasNext()) {
      Map.Entry e = (Map.Entry) itr.next();
      HttpSessionListener listener = (HttpSessionListener) e.getValue();
      listener.sessionDestroyed(se);
    }
  }
  
}
